<?php
// $Id: guppy.php,v 1.71 2004/06/24 20:35:34 jornlind Exp $
// =======================================================================
// Guppy by Jorn Lind-Nielsen (C) 2003.
// ----------------------------------------------------------------------
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WithOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html
// =======================================================================

require_once 'modules/pagesetter/guppy/guppy_common.php';

  // Used by user functions to indicate that Guppy should clear all its output
  // - used with PostNuke for redirecting (pnRedirect) to new pages
global $guppy_outputCleared;
$guppy_outputCleared = false;

/* DEBUGGING
function guppylog($msg)
{
  $f = fopen("/mnt/web_dk/web_domains/fjeldgruppen.dk/tmp/guppy.txt", "a");
  fwrite($f, $msg . " (" . $_SERVER['QUERY_STRING'] . ") \n");
  fflush($f);
  fclose($f);
}
guppylog("\n= GUPPY (" . $_SERVER['QUERY_STRING'] . ") ===\n");
//*/

// =======================================================================
// Command builder
// =======================================================================

class GuppyCommandBuilder
{
  var $data;


  function GuppyCommandBuilder($spec)
  {
    $this->data = null;
    $this->_spec = $spec;
    $this->_closed = false;
    $this->_errors = array();
  }


  function setData($data)
  {
    $this->data = $data;
  }


  function alert($msg)
  {
    echo "<script>alert(\"" . addcslashes($msg, "\0..\37\"\\") . "\");</script>\n";
  }


  function alertAndClose($msg, $url)
  {
    echo "<script>alert(\"" . addcslashes($msg, "\0..\37\"\\") . "\");window.location=\"$url\"</script>\n";
    $this->_closed = true;
  }


  function message($msg)
  {
    echo "<div class=\"guppyMessage\">" . htmlspecialchars($msg) . "</div>\n";
  }


  function errorMessage($msg)
  {
    $this->_errors[] = $msg;
  }


  function insertRow($componentName, $rowIndex, $record) // Inserts at(before) the passed row index
  {
    array_splice($this->data[$componentName]['rows'], $rowIndex, 0, array($record));

    $data = &$this->data[$componentName]['rows'];

    for ($i=count($data)-1; $i>$rowIndex; --$i)
      ++$data[$i]['lineno'];

    $data[$rowIndex]['lineno'] = $rowIndex;
  }


  function deleteRow($componentName, $rowIndex)
  {
    array_splice($this->data[$componentName]['rows'], $rowIndex, 1);

    $data = &$this->data[$componentName]['rows'];

    for ($i=count($data)-1; $i>=$rowIndex; --$i)
      --$data[$i]['lineno'];
  }


  function setRows($componentName, $rows)
  {
    $this->data[$componentName]['rows'] = $rows;
  }


  function setActionEnabling($componentName, $actions)
  {
    $this->data[$componentName]['actions'] = array_join_merge( $this->data[$componentName]['actions'], $actions );
  }


  function treeNewItem($componentName, $rowIndex, $record) // Inserts at(before) the passed row index
  {
    $rows = &$this->data[$componentName]['rows'];

    array_splice($rows, $rowIndex, 0, array($record));

    for ($i=count($rows)-1; $i>$rowIndex; --$i)
      ++$rows[$i]['lineno'];

    $rows[$rowIndex]['lineno'] = $rowIndex;

    if ($rowIndex+1 < count($rows))
      $rows[$rowIndex]['indent'] = $rows[$rowIndex+1]['indent'];
    else
      $rows[$rowIndex]['indent'] = 0;

    $this->_treeCalculateFullTitles($rows, $this->_spec[$componentName]);
  }


    // If $doIndent then indent new item AND append to last element of that level.
  function treeAdditem($componentName, $rowIndex, $record, $doIndent) // Adds after the passed row index
  {
    $rows = &$this->data[$componentName]['rows'];

    $currentIndent = $rows[$rowIndex]['indent'];

      // Find last element at same level (if adding sub-item)
    while ($rowIndex < count($rows)  &&  $rows[$rowIndex+1]['indent'] == $currentIndent+1)
      ++$rowIndex;

    array_splice($rows, $rowIndex+1, 0, array($record));

    for ($i=count($rows)-1; $i>$rowIndex+1; --$i)
      ++$rows[$i]['lineno'];

    $rows[$rowIndex+1]['lineno'] = $rowIndex+1;

    $rows[$rowIndex+1]['indent'] = $currentIndent + ($doIndent ? 1 : 0);

    $this->_treeCalculateFullTitles($rows, $this->_spec[$componentName]);
  }


  function treeChangeItem($componentName, $rowIndex, $record)
  {
    $rows = &$this->data[$componentName]['rows'];

    if ($rowIndex >=0  &&  $rowIndex < count($rows))
    {
      $rows[$rowIndex] = $record;

      $this->_treeCalculateFullTitles($rows, $this->_spec[$componentName]);
    }
  }


  function treeMoveUp($componentName, $rowIndex)
  {
      // A few shorthands
    $rows = &$this->data[$componentName]['rows'];
    $currentIndent = $rows[$rowIndex]['indent'];

    if ($rowIndex <= 0)
      return;
    if ($rows[$rowIndex-1]['indent'] < $currentIndent)
      return;

      // Calculate how far up to move it
    $start = $rowIndex-1;
    while ($start > 0  &&  $rows[$start]['indent'] > $currentIndent)
      --$start;

    $offset = $rowIndex - $start;
    echo "offset: $offset<br>\n";

    for ($i=$rowIndex, $size=count($rows); $i < $size  &&  ($i == $rowIndex  ||  $rows[$i]['indent'] > $currentIndent); ++$i)
    {
      echo "Move $i<br>\n";
      for ($j=0; $j<$offset; ++$j)
      {
        echo "  swap $i-$j = " . ($i-$j) . "<br>\n";
        $tmp = $rows[$i-$j-1];
        $rows[$i-$j-1] = $rows[$i-$j];
        $rows[$i-$j] = $tmp;

        ++$rows[$i-$j]['lineno'];
        --$rows[$i-$j-1]['lineno'];
      }
    }
  }


  function treeMoveDown($componentName, $rowIndex)
  {
      // A few shorthands
    $rows = &$this->data[$componentName]['rows'];
    $size = count($rows);
    $currentIndent = $rows[$rowIndex]['indent'];

    if ($rowIndex >= $size-1)
      return;
    if ($rows[$rowIndex+1]['indent'] < $currentIndent)
      return;

      // Calculate how far to move it - and then move that position up instead of current down
    $start = $rowIndex+1;
    while ($start < $size-1  &&  $rows[$start]['indent'] > $currentIndent)
      ++$start;

    if ($rows[$start]['indent'] > $currentIndent)
      return;

    return $this->treeMoveUp($componentName, $start);
  }


  function treeIndent($componentName, $rowIndex)
  {
      // A few shorthands
    $rows = &$this->data[$componentName]['rows'];
    $currentIndent = $rows[$rowIndex]['indent'];

      // Indent is only allow if there exists at least one item above with the same indent level as the current
    if ($rowIndex == 0)
      $ok = false;
    else
    {
      $ok = false;

      for ($i=$rowIndex-1; $i>=0; --$i)
        if ($rows[$i]['indent'] == $currentIndent)
          $ok = true;
        else if ($rows[$i]['indent'] < $currentIndent)
          break;
    }

    if ($ok)
    {
      $i = $rowIndex;
      $size = count($rows)-1;

        // Indent current item and all below it.
      do
        ++$rows[$i++]['indent'];
      while ($i <= $size  &&  $rows[$i]['indent'] > $currentIndent);

      $this->_treeCalculateFullTitles($rows, $this->_spec[$componentName]);
    }
  }


  function treeUndent($componentName, $rowIndex)
  {
      // Get a few shorthands
    $rows = &$this->data[$componentName]['rows'];
    $currentIndent = $rows[$rowIndex]['indent'];

      // Verify that undent is allowed (may not be moved too far to the left). Not as difficult to check as indent.
    if ($currentIndent > 0)
      $ok = true;
    else
      $ok = false;

    if ($ok)
    {
      $i = $rowIndex;
      $size = count($rows)-1;

        // Undent all below this one
      do
        --$rows[$i++]['indent'];
      while ($i <= $size  &&  $rows[$i]['indent'] > $currentIndent);

      $this->_treeCalculateFullTitles($rows, $this->_spec[$componentName]);
    }
  }


  function treeDelete($componentName, $rowIndex)
  {
      // Get a few shorthands
    $rows = &$this->data[$componentName]['rows'];
    $currentIndent = $rows[$rowIndex]['indent'];

      // Remove current row
    //echo "$rowIndex\n";
    //print_r($rows);
    array_splice($rows, $rowIndex, 1);
    //print_r($rows); exit(0);

      // Update line numbers
    for ($i=count($rows)-1; $i>=$rowIndex; --$i)
      --$rows[$i]['lineno'];

      // Undent all below this one
    $i = $rowIndex;
    $size = count($rows)-1;

    while ($i <= $size  &&  $rows[$i]['indent'] > $currentIndent)
      --$rows[$i++]['indent'];

    $this->_treeCalculateFullTitles($rows, $this->_spec[$componentName]);
  }


  function treeDeleteRecursive($componentName, $rowIndex)
  {
      // Get a few shorthands
    $rows = &$this->data[$componentName]['rows'];
    $currentIndent = $rows[$rowIndex]['indent'];

      // Find last row to delete (not inclusive)
    $lastRowIndex = $rowIndex+1;
    $size = count($rows);
    while ($lastRowIndex < $size  &&  $rows[$lastRowIndex]['indent'] > $currentIndent)
      ++$lastRowIndex;

    $rowsToRemove = $lastRowIndex - $rowIndex;

      // Store data from deleted rows so caller can react on it
    $deletedRows = array();
    for ($i=$rowIndex; $i<$lastRowIndex; ++$i)
      $deletedRows[] = $rows[$i];

      // Remove rows
    array_splice($rows, $rowIndex, $rowsToRemove);

      // Update line numbers
    for ($i=count($rows)-1; $i>=$rowIndex; --$i)
      $rows[$i]['lineno'] -= $rowsToRemove;

    $this->_treeCalculateFullTitles($rows, $this->_spec[$componentName]);

    return $deletedRows;
  }


  function _treeCalculateFullTitles(&$rows, $componentSpec)
  {
    //print_r($componentSpec); exit(0);
    $path = array();
    $titleFieldName = $componentSpec['titleField'];

    for ($i=0,$size=count($rows); $i<$size; ++$i)
    {
      $path[$rows[$i]['indent']] = $rows[$i][$titleFieldName];
      array_splice($path, intval($rows[$i]['indent'])+1);
      $fullTitle = implode(":", $path);

      $rows[$i]['fullTitle'] = $fullTitle;
    }
  }


  function update($componentName, $record, $rowIndex = 0)
  {
    $this->data[$componentName]['rows'][$rowIndex] = $record;

      // Just make sure this is not overwritten
    $this->data[$componentName]['rows'][$rowIndex]['lineno'] = $rowIndex;
  }


  function updatePartial($componentName, $record, $rowIndex = 0)
  {
    $this->data[$componentName]['rows'][$rowIndex] += $record;

      // Just make sure this is not overwritten
    $this->data[$componentName]['rows'][$rowIndex]['lineno'] = $rowIndex;
  }


  function clearOutput()
  {
    global $guppy_outputCleared;
    $guppy_outputCleared = true;
  }


    // Close internal window and redirect to something else afterwards
    // $nextURL may be null to avoid redirection
  function close($nextURL = null)
  {
    $this->_closed = true;
    $this->_nextURL = $nextURL;
    if ($nextURL != null)
      $this->clearOutput();
  }


  function redirect()
  {
    if (isset($this->_nextURL))
      guppy_redirect($this->_nextURL);
  }


  function closeWindow() // Close browser window
  {
    echo "<script>window.close()</script>\n";
    $this->_closed = true;
    $this->clearOutput();
  }


  function openWindow($url, $target=null, $setFocus=false)
  {
    echo "<script>\nvar pagesetterPreviewWindow = window.open('$url'" . ($target == null ? '' : ",'$target'") . ");\n";
    if ($setFocus)
      echo "pagesetterPreviewWindow.focus();\n";
    echo "</script>\n";
  }


  function closed()
  {
    return $this->_closed;
  }


  function hasErrors()
  {
    return count($this->_errors) > 0;
  }


  function generateErrors()
  {
    foreach ($this->_errors as $error)
    {
      $this->alert($error);
    }
  }

  function raw($html)
  {
    echo $html;
  }


  var $_spec;
  var $_closed;
  var $_errors;
  var $_nextURL;
}


// =======================================================================
// Input decoding
// =======================================================================

class GuppyDecodeHandler
{
  var $commander;

  function setCommander(&$commander)
  {
    $this->commander = &$commander;
  }


  function insertRow(&$event)
  {
  }


  function deleteRow(&$event)
  {
  }


  function button(&$event)
  {
  }


  function action(&$event)
  {
  }


  function clickHeader(&$event)
  {
  }


  function menuAction(&$event)
  {
  }


  function treeNewItem(&$event)
  {
      // No authorization - someone else must do this!

    $componentName = $event['action']['component'];
    $rowIndex = $event['action']['rowIndex'];
    $record = $event['data'][$componentName]['record'];
    
    $this->commander->treeNewItem($componentName, $rowIndex, $record);
  }


  function treeAddItem(&$event)
  {
      // No authorization - someone else must do this!

    $componentName = $event['action']['component'];
    $rowIndex = $event['action']['rowIndex'];
    $record = $event['data'][$componentName]['record'];
    
    $this->commander->treeAddItem($componentName, $rowIndex, $record, false);
  }


  function treeAddSubItem(&$event)
  {
      // No authorization - someone else must do this!

    $componentName = $event['action']['component'];
    $rowIndex = $event['action']['rowIndex'];
    $record = $event['data'][$componentName]['record'];
    
    $this->commander->treeAddItem($componentName, $rowIndex, $record, true);
  }


  function treeChangeItem(&$event)
  {
      // No authorization - someone else must do this!
    $componentName = $event['action']['component'];
    $rowIndex = $event['action']['rowIndex'];
    $record = $event['data'][$componentName]['record'];

    $this->commander->treeChangeItem($componentName, $rowIndex, $record);
  }


  function treeMoveUp(&$event)
  {
      // No authorization - someone else must do this!
    $componentName = $event['action']['component'];
    $rowIndex = $event['action']['rowIndex'];
    
    $this->commander->treeMoveUp($componentName, $rowIndex);
  }


  function treeMoveDown(&$event)
  {
      // No authorization - someone else must do this!
    $componentName = $event['action']['component'];
    $rowIndex = $event['action']['rowIndex'];
    
    $this->commander->treeMoveDown($componentName, $rowIndex);
  }


  function treeIndent(&$event)
  {
      // No authorization - someone else must do this!
    $componentName = $event['action']['component'];
    $rowIndex = $event['action']['rowIndex'];
    
    $this->commander->treeIndent($componentName, $rowIndex);
  }


  function treeUndent(&$event)
  {
      // No authorization - someone else must do this!

    $componentName = $event['action']['component'];
    $rowIndex = $event['action']['rowIndex'];
    
    $this->commander->treeUndent($componentName, $rowIndex);
  }


  function treeDelete(&$event)
  {
      // No authorization - someone else must do this!

    $componentName = $event['action']['component'];
    $rowIndex = $event['action']['rowIndex'];
    
    $this->commander->treeDelete($componentName, $rowIndex);
  }


  function treeDeleteRecursive(&$event)
  {
      // No authorization - someone else must do this!

    $componentName = $event['action']['component'];
    $rowIndex = $event['action']['rowIndex'];
    
    $this->commander->treeDeleteRecursive($componentName, $rowIndex);
  }
}


function guppy_decode($decodeHandler)
{
  //echo "<pre>"; print_r($_POST); echo "</pre>\n"; exit(0);

  guppy_beginOutput();

  if (!isset($_POST[':guppyWindow:']))
    return false;

  $windowID = $_POST['windowID'];
  $windowKey = null;

    // Get window data from session
  $windowData = guppy_getWindowData($windowID, $windowKey);
  if ($windowData === false)
  {
    guppy_generateError('guppy_decode', _GUPPYUNKNOWNWINDOWID . '(empty window data)');
    return true;
  }

    // Verify that window data is valid
  $postedWindowKey = $_POST['windowKey'];
  if ($postedWindowKey != $windowKey)
  {
    guppy_generateError(null, _GUPPYUNKNOWNWINDOWID . '(unknown window key)');
    return true;
  }

  //echo "<pre>"; print_r($windowData); echo "</pre>\n"; exit(0);

  $commander = new GuppyCommandBuilder($windowData['spec']['components']);

  $action = guppy_decodeAction($windowData['toolbar']['menuMapping'], $windowData['spec']['components']);
  //echo "<pre>"; print_r($windowData['spec']['components']); echo "</pre>\n"; exit(0);
  //echo "<pre>"; print_r($windowData['data']); echo "</pre>\n";
  $data   = guppy_decodeData($windowData['spec']['components'], $windowData['data'], $commander, $action);
  //echo "<pre>"; print_r($data); echo "</pre>\n"; exit(0);

  $commander->setData($data);

  if (!$commander->hasErrors())
  {
    $result = array( 'action' => $action,
                     'data'   => $data,
                     'extra'  => &$windowData['extra']);

    //echo "<pre>"; print_r($result); echo "</pre>"; exit(0);
    $decodeHandler->setCommander($commander);
    dispatchAction($result, $decodeHandler);

    //echo "<pre>"; print_r($result); echo "</pre>"; exit(0);
    //echo "<pre>"; print_r($commander->data); echo "</pre>\n"; exit(0);
  }
  else
    $result = true;

  if ($commander->closed())
  {
    guppy_deleteWindowID($windowID);
    $commander->redirect();
  }
  else
  {
    $windowData['data'] = $commander->data;
    $windowKey = guppy_getNewWindowKey();

    guppy_dumpHTML($windowID, $windowKey, $windowData['spec'], $windowData['layout']['layout'], 
                   $windowData['toolbar'], $windowData['data'],
                   array( 'actionURL' => $windowData['actionURL'],
                          'windowID'  => $windowID,
                          'action'    => $action) );

    if ($commander->hasErrors())
      $commander->generateErrors();

    guppy_setWindowData($windowID, $windowKey, $windowData);
  }

  return $result;
}


function dispatchAction(&$result, $decodeHandler)
{
  $actionKind = $result['action']['kind'];

  switch ($actionKind)
  {
    case 'insertRow':
      $decodeHandler->insertRow($result);
    break;

    case 'deleteRow':
      $decodeHandler->deleteRow($result);
    break;

    case 'button':
      $decodeHandler->button($result);
    break;

    case 'action':
      $decodeHandler->action($result);
    break;

    case 'clickHeader':
      $decodeHandler->clickHeader($result);
    break;

    case 'menuAction':
      $decodeHandler->menuAction($result);
    break;

    case 'treeNewItem':
      $decodeHandler->treeNewItem($result);
    break;

    case 'treeAddItem':
      $decodeHandler->treeAddItem($result);
    break;

    case 'treeAddSubItem':
      $decodeHandler->treeAddSubItem($result);
    break;

    case 'treeChangeItem':
      $decodeHandler->treeChangeItem($result);
    break;

    case 'treeMoveUp':
      $decodeHandler->treeMoveUp($result);
    break;

    case 'treeMoveDown':
      $decodeHandler->treeMoveDown($result);
    break;

    case 'treeIndent':
      $decodeHandler->treeIndent($result);
    break;

    case 'treeUndent':
      $decodeHandler->treeUndent($result);
    break;

    case 'treeDelete':
      $decodeHandler->treeDelete($result);
    break;

    case 'treeDeleteRecursive':
      $decodeHandler->treeDeleteRecursive($result);
    break;  
  }
}


function guppy_decodeData($spec, $oldData, &$commander, $action)
{
  $data = array();

  foreach ($spec as $componentName => $componentSpec)
  {
    $validate = ($action['kind'] != 'button'  ||  $action['buttonKind'] != 'cancel');

    if ($componentSpec['kind'] == 'card')
    {
      $rows = array( guppy_decodeRecord($componentName, $componentSpec, 0, $commander, $validate) );

      $data[$componentName] = array_join_merge($oldData[$componentName], array( 'rows' => $rows ));
    }
    else
    if ($componentSpec['kind'] == 'table')
    {
      $rows = array();
      $rowCount = intval($_POST["s:$componentName"]);

      for ($rowIndex=0; $rowIndex<$rowCount; ++$rowIndex)
      {
        $rows[] = guppy_decodeRecord($componentName, $componentSpec, $rowIndex, $commander, $validate);
      }

      $data[$componentName] = array_join_merge($oldData[$componentName], array( 'rows' => $rows ));
    }
    else
    if ($componentSpec['kind'] == 'tree')
    {
      $action = $_POST['action'];
      $validate = (    $action == 'treeNewItem'  ||  $action == 'treeAddItem'
                   ||  $action == 'treeAddSubItem'  ||  $action == 'treeChangeItem');

      $record = guppy_decodeRecord($componentName, $componentSpec, 0, $commander, $validate);

      if ($action == 'treeChangeItem')
      {
        $rowIndex = intval($_POST['rowIndex']);
        $record = $record + $oldData[$componentName]['rows'][$rowIndex];
      }

      $data[$componentName] = array('rows' => $oldData[$componentName]['rows'], 'record' => $record);
    }
  }

  return $data;
}


function guppy_decodeRecord($componentName, $componentSpec, $rowIndex, &$commander, $doValidate)
{
  $record = array();

  foreach ($componentSpec['fields'] as $field)
  {
    if ($field['inUse'])
    {
      $fieldName  = $field['name'];
      $fieldTitle = isset($field['title']) ? $field['title'] : $fieldName;
      $fieldType  = $field['type'];
      $mandatory  = $field['mandatory'];
      $formName   = "f:$componentName:$fieldName:$rowIndex";

      if ($fieldType == 'bool')
      {
        $fieldDataRaw = isset($_POST[$formName]);
      }
      else
      if ($fieldType == 'radio')
      {
        $formName = "f:$componentName:$fieldName";
        $fieldDataRaw = ($_POST[$formName] == $rowIndex ? true : false);
      }
      else
      if ($fieldType == 'select')
      {
        $selectIndex = intval($_POST[$formName]);
        $size = count($field['options']);

        if ($selectIndex > 0  &&  $selectIndex < $size)
        {
          $fieldDataRaw = $field['options'][$selectIndex]['value'];
        }
        else
        {
          $fieldDataRaw = NULL;

          if ($mandatory  &&  $doValidate)
            $commander->errorMessage(_GUPPYMISSINGMANDATORY . ': ' . guppy_Translate($fieldTitle));
        }
      }
      else
      {
        $fieldDataRaw = $_POST[$formName];

        if ($mandatory  &&  $fieldDataRaw == ''  &&  $doValidate)
          $commander->errorMessage(_GUPPYMISSINGMANDATORY. ': ' . guppy_Translate($fieldTitle));
        else
        if ($fieldDataRaw == '')
          $fieldDataRaw = null;
        else
        {
            // If magic quotes are on then all query/post variables are escaped - so strip slashes
          if (get_magic_quotes_gpc())
            $fieldDataRaw = stripslashes($fieldDataRaw);

          if ($fieldType == 'datetime')
          {
              // Scan input as date/time and rewrite into correct format
            $datetime = strtotime($fieldDataRaw);
            if ($datetime === -1)
              $commander->errorMessage(_GUPPYINVALIDDATETIME . ': ' . guppy_Translate($fieldTitle));
            $fieldDataRaw = strftime("%Y-%m-%d %H:%M:%S", $datetime);
          }
          else
            guppy_validateField($fieldDataRaw, $fieldTitle, $fieldType, $commander);
        }
      }

      $record[$fieldName] = $fieldDataRaw;
    }
  }
  
  return $record;
}


function guppy_validateField($fieldDataRaw, $fieldTitle, $fieldType, &$commander)
{
  if ($fieldType == 'int')
  {
    if (!str_is_int($fieldDataRaw))
      $commander->errorMessage(_GUPPYINVALIDINT . ': ' . guppy_Translate($fieldTitle));
  }
  else if ($fieldType == 'real')
  {
    if (!is_numeric($fieldDataRaw))
      $commander->errorMessage(_GUPPYINVALIDREAL . ': ' . guppy_Translate($fieldTitle));
  }
  else if ($fieldType == 'date')
  {
    list ($year,$month,$day) = split('-', $fieldDataRaw);

    if (!checkdate($month,$day,$year))
      $commander->errorMessage(_GUPPYINVALIDDATE . ': ' . guppy_Translate($fieldTitle));
  }
  else if ($fieldType == 'time')
  {
    list ($hour,$min,$sec) = split(':', $fieldDataRaw);

    if ($hour < 0  ||  $hour > 23  ||  $min < 0  ||  $min > 59  ||  $sec < 0  ||  $sec > 59)
      $commander->errorMessage(_GUPPYINVALIDTIME . ': ' . guppy_Translate($fieldTitle));
  }
}


function str_is_int($str) 
{
  $var = intval($str);
  return ("$str" == "$var");
}


function guppy_decodeAction($menuMapping, &$componentSpecs)
{
  $action     = $_POST['action'];
  $component  = $_POST['component'];
  $rowIndex   = intval($_POST['rowIndex']);

  if ($action == 'insertRow'  ||  $action == 'deleteRow')
  {
    return array( 'kind'      => $action,
                  'component' => $component,
                  'rowIndex'  => $rowIndex );
  }
  else
  if ($action == 'button')
  {
    $button = $_POST['param1'];
    $buttonKind = $componentSpecs[$component]['actions'][$button]['kind'];

    return array( 'kind'       => $action,
                  'component'  => $component,
                  'button'     => $button,
                  'buttonKind' => $buttonKind );
  }
  else
  if ($action == 'action')
  {
    return array( 'kind'      => $action,
                  'component' => $component,
                  'rowIndex'  => $rowIndex,
                  'action'    => $_POST['param1'] );
  }
  else
  if ($action == 'clickHeader')
  {
    return array( 'kind'      => $action,
                  'component' => $component,
                  'column'    => $_POST['param1'] );
  }
  else
  if ($action == 'menuAction')
  {
    return array( 'kind'      => $action,
                  'action'    => $menuMapping["$component:$rowIndex"] );
  }
  else
  if (   $action == 'treeNewItem'  || $action == 'treeChangeItem'  || $action == 'treeAddItem'  ||  $action == 'treeAddSubItem'
      || $action == 'treeMoveUp' ||  $action == 'treeMoveDown'  || $action == 'treeIndent' ||  $action == 'treeUndent'  
      || $action == 'treeDelete'  ||  $action == 'treeDeleteRecursive' || $action == 'treeInsert'  ||  $action == 'treeAdd'  
      ||  $action == 'treeAddSub'  ||  $action == 'treeEdit')
  {
    return array( 'kind'      => $action,
                  'component' => $component,
                  'rowIndex'  => $rowIndex );
  }

  return array( 'kind' => 'unknown' );
}


// =======================================================================
// Output generation
// =======================================================================

function guppy_open($args)
{
  guppy_beginOutput();

    // Get specification as string
  if (isset($args['specFile']))
  {
    if (($spec=guppy_loadfile('guppy_open', $args['specFile'])) === false)
      return false;
  }
  else if (isset($args['spec']))
  {
    $spec = $args['spec'];
  }
  else if (isset($args['rawSpec']))
  {
    $rawSpec = $args['rawSpec'];
  }
  else
    return guppy_generateError('guppy_open', "Missing 'spec', 'rawSpec' or 'specFile' setting");

    // Get layout as string
  if (isset($args['layoutFile']))
  {
    if (($layout=guppy_loadfile('guppy_open', $args['layoutFile'])) === false)
      return false;
  }
  else if (isset($args['layout']))
  {
    $layout = $args['layout'];
  }
  else if (isset($args['rawLayout']))
  {
    $rawLayout = $args['rawLayout'];
  }
  else
    return guppy_generateError('guppy_open', "Missing 'layout', 'rawLayout' or 'layoutFile' setting");

    // Get toolbar as string
  if (isset($args['toolbarFile']))
  {
    if (($toolbar=guppy_loadfile('guppy_open', $args['toolbarFile'])) === false)
      return false;
  }
  else if (isset($args['toolbar']))
  {
    $toolbar = $args['toolbar'];
  }
  else if (isset($args['rawToolbar']))
  {
    $rawToolbar = $args['rawToolbar'];
  }
  else
    $rawToolbar = null;

    // Get data
  if (isset($args['data']))
  {
    $data = $args['data'];
  }
  else
    return guppy_generateError('guppy_open', "Missing 'data' value");

  $extra = $args['extra'];

    // Get form action
  if (isset($args['actionURL']))
  {
    $actionURL = $args['actionURL'];
  }
  else
    return guppy_generateError('guppy_open', "Missing 'actionURL' value");

    // Dynamically load the parser functions for spec and layout
  require_once 'modules/pagesetter/guppy/guppy_parser.php';

    // Parse the spec and layout - if supplied as XML data
  if ($spec)
  {
    guppy_parseXMLSpec($spec, $args['options']);
    global $guppyParsedSpec;
    $spec = &$guppyParsedSpec;  // Must be copy by reference
  }
  else
    $spec = $rawSpec;

  if ($layout)
    $layout = guppy_parseXMLLayout($layout);
  else
    $layout = $rawLayout;

  if ($toolbar)
    $toolbar = guppy_parseXMLToolbar($toolbar);
  else
    $toolbar = $rawToolbar;

  guppy_initializeData($spec, $data);

  $windowID = guppy_getNewWindowID();
  $windowKey = guppy_getNewWindowKey();


  if (isset($args['editorsetup']))
    echo "<script src=\"$args[editorsetup]\"></script>\n";

  guppy_dumpHTML($windowID, $windowKey, $spec, $layout['layout'], $toolbar, $data,
                 array( 'actionURL' => $actionURL,
                        'windowID'  => $windowID,
                        'action'    => array() ) );

  $windowData = array( 'spec'      => $spec,
                       'layout'    => $layout,
                       'toolbar'   => $toolbar,
                       'data'      => $data,
                       'extra'     => $extra,
                       'actionURL' => $actionURL);

  guppy_setWindowData($windowID, $windowKey, $windowData);
}


function guppy_initializeData(&$spec, &$data)
{
  foreach ($spec['components'] as $componentName => $componentSpec)
  {
    if (isset($data[$componentName]['actions']))
      $actionsData = &$data[$componentName]['actions'];
    else
      $actionsData = array();

    foreach ($componentSpec['actions'] as $actionName => $actionSpec)
    {
        // All actions enabled initially unless otherwised specified
      if (!($actionsData[$actionName] === false))
        $actionsData[$actionName] = true;
    }

    $data[$componentName]['actions'] = $actionsData;
  }
}


function guppy_dumpHTML($windowID, $windowKey, $spec, $layout, &$toolbar, $data, $pageInfo)
{
  global $guppyBaseURL;

  echo "\n<script src=\"$guppyBaseURL/guppy.js\"></script>\n";
  echo "<form action=\"$pageInfo[actionURL]\" method=\"post\" id=\"guppyForm\">\n";
  echo "<input type=\"hidden\" name=\":guppyWindow:\" value=\"1\"/>\n";
  echo "<input type=\"hidden\" name=\"windowID\" value=\"$windowID\"/>\n";
  echo "<input type=\"hidden\" name=\"windowKey\" value=\"" . $windowKey . "\"/>\n";
  echo "<input type=\"hidden\" name=\"action\" value=\"\"/>\n";
  echo "<input type=\"hidden\" name=\"component\" value=\"\"/>\n";
  echo "<input type=\"hidden\" name=\"rowIndex\" value=\"\"/>\n";
  echo "<input type=\"hidden\" name=\"param1\" value=\"\"/>\n";

  echo "<div class=\"guppy\">\n";
  if ($toolbar != null)
  {
    $toolbar['menuMapping'] = array();
    guppy_buildToolbar($spec, $toolbar, $toolbar['menuMapping']);
  }
  guppy_buildLayout($spec, $layout, $data, $pageInfo);
  echo "</div>\n";
  echo "</form>\n";
}


/*========================================================================
  Output generator
========================================================================*/

function guppy_buildToolbar($spec, $toolbar, &$menuMapping)
{
  //echo "<pre>"; print_r($spec); echo "</pre>"; exit(0);
  //echo "B<pre>"; print_r($toolbar); echo "</pre>\n"; exit(0);

    // Make sure menu code is included
  global $guppyBaseURL;
  echo "<script src=\"$guppyBaseURL/psmenu.js\"></script>\n";

  if (count($toolbar['menu']) > 0)
  {
    echo "<table class=\"toolbar\">\n";
    echo "<tr><td width=\"100%\">\n<table class=\"menu\"><tr>";
    $id = 0;
    foreach ($toolbar['menu'] as $menu)
    {
      $title = guppy_translate($menu['title']);

      echo "<td class=\"normal\" onMouseOver=\"menuTopMouseOver(this)\" onMouseOut=\"menuTopMouseOut(this)\" onMouseDown=\"mainmenu.popup(this,event,'$id')\">$title</td>\n";

      ++$id;
    }

    echo "</tr></table></td></tr>\n";
    echo "</table>\n";

    $id = 0;
    foreach ($toolbar['menu'] as $menu)
    {
      $subMenu = $menu['subMenu'];

      guppy_buildMenuList($subMenu, $id, $menuMapping);

      ++$id;
    }
  }
}


function guppy_buildMenuList($menu, $id, &$menuMapping)
{
    // Menu HTML setup
  echo "<div class=\"psmenu\" id=\"menu-$id\" onmouseout=\"psmenu.onMouseOutDiv(this)\">\n<table>\n";

  $index = 0;
  foreach ($menu as $menuItem)
  {
    $title = guppy_translate($menuItem['title']);

    echo "<tr class=\"psmenu-menuItem\" onmouseover=\"psmenu.onMouseOver(this)\" onmouseout=\"psmenu.onMouseOut(this)\""
         . " onmousedown=\"psmenu.onMouseDown(this)\">\n"
         . "<td class=\"psmenu-title\"><nobr>$title</nobr></td>\n"
         . "</tr>\n";

    $menuMapping["menu-$id:$index"] = $menuItem['action'];
    ++$index;
  }

    // End menu HTML
  echo "</table>\n</div>\n";
}


function guppy_buildLayout($spec, $layout, $data, $pageInfo)
{
  //echo "<pre>"; print_r($spec); echo "</pre><br><pre>"; print_r($layout); echo "</pre>\n"; exit(0);

  echo "\n<table class=\"guppy-pane\">\n";
  
  foreach ($layout as $row)
  {
    guppy_buildRow($spec, $row, $data, 0, $pageInfo);
  }

  echo "</table>\n";
}


function guppy_buildRow($spec, $row, $data, $rowIndex, $pageInfo)
{
  echo "<tr>\n";
  
  foreach ($row as $element)
  {
    guppy_buildElement($spec, $element, $data, $rowIndex, $pageInfo, false /* not inTable */);
  }

  echo "</tr>\n";
}


function guppy_buildElement($spec, $element, $data, $rowIndex, $pageInfo, $inTable)
{
  $kind = $element['kind'];

  switch ($kind)
  {
    case 'component':
      $ok = guppy_buildComponent($spec, $element, $data, $pageInfo); 
    break;

    case 'island':
      $ok = guppy_buildIsland($spec, $element, $data, $pageInfo); 
    break;

    case 'title':
      $ok = guppy_buildTitle($spec, $element, $data); 
    break;

    case 'input':
      $ok = guppy_buildField($spec, $element, $data, $rowIndex, $inTable); 
    break;

    case 'action':
      $ok = guppy_buildAction($spec, $element, $data, $rowIndex); 
    break;

    case 'raw':
      $ok = guppy_buildRaw($spec, $element, $data, $rowIndex); 
    break;

    /* Unused
    case 'treeParent':
      $ok = guppy_buildTreeParent($pageInfo); 
    break;
    */
    default:
      return guppy_generateError('element', "unknown element kind '$kind'");
    break;
  }

  return $ok;
}


function guppy_startTableCell($element, $fieldSpec, $inTable)
{
  $attribs = "";
  if (isset($element['colspan']))
    $attribs .= " colspan=\"$element[colspan]\"";

  if ($inTable  &&  ($fieldSpec['type'] == 'bool'  ||  $fieldSpec['type'] == 'radio'))
    $attribs .= " align=\"center\"";
  
  echo "<td$attribs>";
}


function guppy_endTableCell($element, $fieldSpec)
{
  echo "</td>\n";
}


function guppy_buildComponent($spec, $component, $data, $pageInfo)
{
    // Check existence of component name and get it
  if (!isset($component['name']))
    return guppy_generateError("component", "missing component name");
  $componentName = $component['name'];

    // Check existence of component spec and get it
  if (!isset($spec['components'][$componentName]))
    return guppy_generateError("component $componentName", "unknown component");
  $componentSpec = $spec['components'][$componentName];

    // Check existence of component kind and get it
  if (!isset($componentSpec['kind']))
    return guppy_generateError("component $componentName", "missing component kind");
  $componentKind = $componentSpec['kind'];

    // Check existence of component title and get it
  if (!isset($componentSpec['title']))
    return guppy_generateError("component $componentName", "missing component title");
  $componentTitle = $componentSpec['title'];
  $componentTitle = guppy_translate($componentTitle);

    // Check existence of layout and get it
  if (!isset($component['layout']))
    return guppy_generateError("component $componentName", "missing component layout");
  $componentLayout  = $component['layout'];

    // Check existence of component data and get it
  if (!isset($data[$componentName]))
     return guppy_generateError("component $componentName", "missing component data");
  $componentData = $data[$componentName];

  if (!isset($componentData['rows']))
     return guppy_generateError("component $componentName", "missing component data rows");
  $componentDataRows = $componentData['rows'];

  guppy_startTableCell($component, array(), false);

  echo "<div class=\"component\" id=\"$componentName\">\n";
  echo "<table class=\"component\">\n";

    // Print component title
  echo "<tr><td class=\"header\"><table class=\"ctab\"><tr><td class=\"ctab1\"> </td><td class=\"ctab2\">\n";
  echo "<nobr>" . htmlspecialchars($componentTitle) . "</nobr>";
  echo "</td><td class=\"ctab3\"> </td><td class=\"ctab4\"></td></tr></td></tr></table></tr>\n";

    // Add top buttons
  echo "<tr><td class=\"buttons\">\n";
  guppy_buildButtons($componentName, $componentSpec['actions'], $component['buttonsTop'], $componentData);
  echo "</td></tr>\n";

    // Insert component pane (main input area)
  echo "<tr><td class=\"content\">\n";
  if ($componentKind == 'card')
  {
    if (!count($componentDataRows) == 1)
       return guppy_generateError("component $componentName", "component data does not have exactly one row");
    $recordData = $data[$componentName]['rows'][0];

    echo "<div class=\"card\">\n";
    $ok = guppy_buildLayout($componentSpec, $componentLayout, $recordData, $pageInfo);
    echo "</div>\n";
  }
  else if ($componentKind == 'table')
  {
    $ok = guppy_buildTable($componentSpec, $componentLayout, $componentDataRows, $pageInfo);
  }
  else if ($componentKind == 'tree')
  {
    $ok = guppy_buildTree($componentSpec, $componentLayout, $componentDataRows, $pageInfo);
  }
  else
     return guppy_generateError("component $componentName", "unknown component kind '$componentKind'");
  echo "</td></tr>\n";

    // Add bottom buttons
  echo "<tr><td class=\"buttons\">\n";
  guppy_buildButtons($componentName, $componentSpec['actions'], $component['buttonsBottom'], $componentData);
  echo "</td></tr>\n";

  echo "</table>\n";

  guppy_endTableCell($component, array());

  return $ok;
}



function guppy_buildTable($componentSpec, $componentLayout, $componentDataRows, $pageInfo)
{
  global $guppyThemeURL;

  $insertIconHTML = "<img class=\"clickable\" src=\"$guppyThemeURL/insert.gif\" onClick=\"handleOnClickInsertRow(this)\">";
  $deleteIconHTML = "<img class=\"clickable\" src=\"$guppyThemeURL/delete.gif\" onClick=\"handleOnClickDeleteRow(this)\">";

  $useInsertRow = $componentSpec['rowOperations']['insert'];
  $useDeleteRow = $componentSpec['rowOperations']['delete'];

  $componentName = $componentSpec['name'];

    // Setup insert/delete confirmation handlers
  echo "<script>\n";
  echo "var confirmInsert" . $componentName . " = \""
       . (isset($componentSpec['rowOperations']['insertConfirm'])
          ? addcslashes(guppy_translate($componentSpec['rowOperations']['insertConfirm']), "\0..\37\"")
          : "") 
       . "\";\n";
  echo "var confirmDelete" . $componentName . " = \""
       . (isset($componentSpec['rowOperations']['deleteConfirm'])
          ? addcslashes(guppy_translate($componentSpec['rowOperations']['deleteConfirm']), "\0..\37\"")
          : "") 
       . "\";\n";
  echo "</script>\n";

  echo "<input type=\"hidden\" name=\"s:$componentSpec[name]\" value=\"" . count($componentDataRows) . "\"/>\n";
  echo "<table class=\"grid\">\n";

    // Create headings
  echo "<tr>\n";
  if ($useInsertRow)
    echo "<th>$insertIconHTML</th>\n";
  if ($useDeleteRow)
    echo "<th></th>"; 
  foreach ($componentLayout as $field)
  {
    $fieldKind = $field['kind'];
    $fieldName = $field['name'];
    
    if ($fieldKind == 'input')
    {
      if (!isset($componentSpec['fields'][$fieldName]))
        return guppy_generateError("$componentSpec[name]", "Unknown field '$fieldName'");
      $fieldSpec = $componentSpec['fields'][$fieldName];
      $fieldTitle = (isset($field['title']) ? $field['title'] : $fieldSpec['title']);
    }
    else
    {
      if (!isset($componentSpec['actions'][$fieldName]))
        return guppy_generateError("$componentSpec[name]", "Unknown action '$fieldName'");
      $actionSpec = $componentSpec['actions'][$fieldName];
      $fieldTitle = (isset($field['title']) ? $field['title'] : $actionSpec['title']);
    }
    $fieldTitle = guppy_translate($fieldTitle);
    
    $style = "";

    if (isset($field['width']))
      $style = "width: $field[width];";

    if ($style != "")
      $style = " style=\"$style\"";

    if ($componentSpec['clickOnHeaders']  &&  $fieldKind == 'input')
      $fieldTitle = "<span onClick=\"handlOnClickHeader('$fieldName','$componentName')\" class=\"clickable action\">"
                    . htmlspecialchars($fieldTitle) . "</span>";
    
    echo "<th$style>$fieldTitle</th>";
  }
  echo "\n</tr>\n";

    // If this table uses lineno control then sort the data by lineno before showing
    // (could also have assumed this was done by caller - don't really know what is best)
  if ($componentSpec['lineno'])
    usort($componentDataRows, "guppy_tableLinenoCmp");

    // Create the rows
  $rowIndex = 0;
  foreach ($componentDataRows as $record)
  {
    echo "<tr>\n";

      // Add ins/del icons
    if ($useInsertRow)
      echo "<td>$insertIconHTML</td>\n";
    if ($useDeleteRow)
      echo "<td>$deleteIconHTML</td>\n";

    foreach ($componentLayout as $field)
    {
      guppy_buildElement($componentSpec, $field, $record, $rowIndex, $pageInfo, true /* in table*/);
    }
    echo "</tr>\n";

    ++$rowIndex;
  }

  echo "</table>\n";
}


function pagesetterGetTreeMenu()
{
  global $guppyThemeURL;

    // Must match menu sequence in guppy.js
  $menu = array( array( 'title' => _GUPPYEDIT,            'image' => 'edit.gif'  ), 
                 array( 'title' => _GUPPYADD,             'image' => 'treeadd.gif' ), 
                 array( 'title' => _GUPPYADDTREESUBITEM,  'image' => 'treeaddsubitem.gif' ), 
                 array( 'title' => _GUPPYINSERT,          'image' => 'treeinsert.gif' ), 
                 array( 'title' => _GUPPYDELETE,          'image' => 'delete.gif' ), 
                 array( 'title' => _GUPPYMOVEUP,          'image' => 'moveup.gif' ), 
                 array( 'title' => _GUPPYMOVEDOWN,        'image' => 'movedown.gif' ), 
                 array( 'title' => _GUPPYINDENT,          'image' => 'indent.gif' ), 
                 array( 'title' => _GUPPYUNDENT,          'image' => 'undent.gif' ),
                 array( 'title' => _GUPPYDELETERECURSIVE, 'image' => 'deleteall.gif' ) );
    
    // Menu HTML setup
  $menuHTML = "<div class=\"psmenu\" id=\"psmenu-tree\" onmouseout=\"psmenu.onMouseOutDiv(this)\">\n"
              . "<table>\n";

    // One table row for each menu item
  foreach ($menu as $menuItem)
    $menuHTML .= "<tr class=\"psmenu-menuItem\" onmouseover=\"psmenu.onMouseOver(this)\" onmouseout=\"psmenu.onMouseOut(this)\""
                 . " onmousedown=\"psmenu.onMouseDown(this)\">\n"
                 . "<td class=\"psmenu-icon\"><img src=\"$guppyThemeURL/$menuItem[image]\"></td>"
                 . "<td class=\"psmenu-title\"><nobr>$menuItem[title]</nobr></td>\n"
                 . "</tr>\n";

    // End menu HTML
  $menuHTML .= "</table>\n"
               . "</div>\n";

  return $menuHTML;
}


function guppy_buildTree($componentSpec, $componentLayout, $componentDataRows, $pageInfo)
{
  $componentName = $componentSpec['name'];

  if (!isset($componentSpec['titleField']))
    return guppy_generateError("$componentName", "Missing 'titleField' attribute in tree component layout");
  $titleFieldName = $componentSpec['titleField'];

  if (!isset($componentSpec['fields'][$titleFieldName]))
    return guppy_generateError("$componentName", "Unknown field '$titleFieldName'");

    // Setup
  $actionKind = $pageInfo['action']['kind'];
  if ($actionKind == 'treeEdit')
  {
    $editComponent = $pageInfo['action']['component']; 
    $currentRowIndex = $pageInfo['action']['rowIndex'];
    $legend = _GUPPYEDITTREEITEM;
  }
  else
  if ($actionKind == 'treeInsert')
  {
    $editComponent = null;
    $currentRowIndex = $pageInfo['action']['rowIndex'];
    $legend = _GUPPYINSERTTREEITEM;
  }
  else if ($actionKind == 'treeAdd')
  {
    $editComponent = null;
    $currentRowIndex = $pageInfo['action']['rowIndex'];
    $legend = _GUPPYADDTREEITEM;
  }
  else if ($actionKind == 'treeAddSub')
  {
    $editComponent = null;
    $currentRowIndex = $pageInfo['action']['rowIndex'];
    $legend = _GUPPYADDTREESUBITEM;
  }
  else
  {
    $editComponent = null;
    $currentRowIndex = count($componentDataRows);
    $legend = _GUPPYNEWTREEITEM;
  }

    // Build menu
  echo pagesetterGetTreeMenu();

  echo "<div class=\"tree-component\">\n";

  echo "<table class=\"tree\">\n";

    // Create the tree view
  $rowIndex = 0;

  foreach ($componentDataRows as $record)
  {
    echo "<tr>\n";
    
    $isCurrent = ($rowIndex == $currentRowIndex);
    guppy_buildTreeRow($componentName, $record['lineno'], $record['indent'], $record['id'], $record[$titleFieldName], $isCurrent);
    
    echo "</tr>\n";

    ++$rowIndex;
  }

  echo "</table>\n";

  $pageInfo['parentSelectorHTML'] = $parentSelectorHTML;

    // Make edit area

  echo "<fieldset>";
  echo "<legend>$legend</legend>\n";
  if ($actionKind == 'treeEdit')
  {
    $ok = guppy_buildLayout($componentSpec, $componentLayout, $componentDataRows[$currentRowIndex], $pageInfo);
    $action = 'edit';
  }
  else
  {
    $ok = guppy_buildLayout($componentSpec, $componentLayout, array(), $pageInfo);
  
    if ($actionKind == 'treeAdd')
      $action = 'add';
    else if ($actionKind == 'treeAddSub')
      $action = 'addSub';
    else
      $action = 'new';
  }

  echo "<button onClick=\"handleOnClickTreeSubmit('$componentName', '$currentRowIndex', '$action')\">" . _GUPPYSUBMIT . "</button>\n";
  echo "<button onClick=\"handleOnClickTreeCancel()\">" . _GUPPYCANCEL . "</button>\n";
  echo "</fieldset>\n";

    // Make sure focus is set to first field of the edit area
  $titleFieldName = $componentSpec['titleField'];
  $formFieldName = "f:$componentName:$titleFieldName:0";
  echo   "<script>var guppyInitialFocusElement = '$formFieldName';\n"
       . "window.onload = guppyHandleOnLoad;" . "</script>\n";

  echo "</div>\n";
  return $ok;
}


function guppy_buildTreeRow($componentName, $lineno, $indent, $rowID, $title, $isCurrent)
{
  global $guppyThemeURL;

  if ($isCurrent)
  {
    $menuItemHTML = "<img src=\"$guppyThemeURL/menuitem.gif\"\">";
    $classHTML = " class=\"selected\"";
  }
  else
  {
    $menuItemHTML = "<img src=\"$guppyThemeURL/menuitem.gif\"\">";
    $classHTML = "";
  }

  echo "<td$classHTML>";

  echo   "<div class=\"clickable\" style=\"padding-left: " . ($indent*10) . "px;\" "
       . "onmousedown=\"treeMenu.onMouseDown('$componentName', $lineno, event, 'psmenu-tree')\">"
       . "$menuItemHTML $title" . ($rowID ? " ($rowID)" : '') //. " [$lineno,$indent]"
       . "</div>";
  
  echo "</td>\n";
}


function guppy_tableLinenoCmp($a, $b)
{
  if ($a['lineno'] == $b['lineno']) // should not be possible ... but you never know :-)
    return 0;

  if ($a['lineno'] < $b['lineno'])
    return -1;

  return 1;
}


function guppy_buildButtons($componentName, &$actionSpec, &$buttonsLayout, &$componentData)
{
  if (count($buttonsLayout) == 0)
    return;
  
  echo "<table class=\"buttons\"><tr>\n";
  foreach ($buttonsLayout as $button)
  {
    if ($button['kind'] == 'button')
      guppy_buildButton($componentName, $actionSpec, $button, $componentData);
    else
      guppy_buildButtonGroup($componentName, $actionSpec, $button, $componentData);
  }
  echo "</tr></table>\n";
}


function guppy_buildButton($componentName, &$actionSpec, &$button, &$componentData)
{
  $buttonName = $button['name'];
  if (!isset($actionSpec[$buttonName]))
    return guppy_generateError("$componentName", "Unknown button '$buttonName'");
  $action = $actionSpec[$buttonName];
  $buttonTitle = (isset($button['title']) ? $button['title'] : $action['title']);
  $buttonTitle = guppy_translate($buttonTitle);
  $buttonHint = (isset($button['hint']) ? $button['hint'] : $action['hint']);
  $buttonHint = guppy_translate($buttonHint);
  $buttonFormName = "b:$componentName:$buttonName";

  if (!$componentData['actions'][$buttonName])
    $disabled = " disabled=\"1\"";
  else
    $disabled = '';

  if (isset($buttonHint))
    $title = " title=\"$buttonHint\"";
  else
    $title = '';

  echo   "<td><button name=\"$buttonFormName\" onClick=\"handleOnClickButton('$componentName','$buttonName')\"$disabled$title>"
       . htmlspecialchars($buttonTitle) . "</button></td>\n";
}


function guppy_buildButtonGroup($componentName, &$actionSpec, &$buttonGroup, &$componentData)
{
  $groupID   = guppy_getElementID();
  $groupTitle = $buttonGroup['title'];
  $groupTitle = guppy_translate($groupTitle);

  echo   "<td>$groupTitle:\n"
       . "<select id=\"$groupID\" onChange=\"handleOnChangeButtonGroup('$groupID')\">\n";

  $initialTitle = null;

  foreach ($buttonGroup['buttons'] as $button)
  {
    $buttonName = $button['name'];
    if (!isset($actionSpec[$buttonName]))
      return guppy_generateError("$componentName", "Unknown button '$buttonName'");

      // Is button enabled?
    if ($componentData['actions'][$buttonName])
    {
      $action = $actionSpec[$buttonName];
      $buttonTitle = (isset($button['title']) ? $button['title'] : $action['title']);
      $buttonTitle = guppy_translate($buttonTitle);

      if (isset($action['hint']))
        $title = " title=\"" . guppy_Translate($action['hint']) . "\"";
      else
        $title = '';

      if ($initialTitle == null)
        $initialTitle = guppy_Translate($action['hint']);

      echo "<option value=\"$buttonName\"$title>" . htmlspecialchars($buttonTitle) . "</option>\n";
    }
  }

  echo  "</select>\n"
      . "<button id=\"b:$groupID\" onClick=\"handleOnClickButtonGroup('$componentName','$groupID')\" "
      . "title=\"" . htmlspecialchars($initialTitle) . "\">" . _GUPPYACTIONGO . "</button>\n"
      . "</td>\n";
}


function guppy_buildIsland($spec, $element, $data, $pageInfo)
{
  guppy_startTableCell($element, array(), false);

  $title = guppy_translate($element['title']);
  echo "<fieldset><legend>" . htmlspecialchars($title) . "</legend>\n";

  guppy_buildLayout($spec, $element['layout'], $data, $pageInfo);

  echo "</fieldset>\n";

  guppy_endTableCell($element, array());
}


function guppy_buildTitle($componentSpec, $element, $data)
{
    // Check for existence of field name and get it
  if (isset($element['name']))
  {
    $fieldName = $element['name'];

      // Check for existence of field spec in component spec and get it
    if (!isset($componentSpec['fields'][$fieldName]))
      return guppy_generateError("$componentSpec[name]", "undefined field '$fieldName'");
    $fieldSpec = $componentSpec['fields'][$fieldName];
  }

    // Get optional field title (first from layout then from field spec)
  if (isset($element['title']))
    $fieldTitle = $element['title'];
  else if (isset($fieldSpec['title']))
    $fieldTitle = $fieldSpec['title'];
  else
    $fieldTitle = NULL;

    // If a title is assigned then print it together with the field
  guppy_startTableCell($element, array(), false);

  if ($fieldTitle != NULL)
  {
    $fieldTitle = guppy_translate($fieldTitle);
    echo "<span class=\"tl\">" . htmlspecialchars($fieldTitle) . "</span>";
  }

  if (!empty($fieldSpec['hint']))
  {
    global $guppyThemeURL;

    echo "&nbsp;<img src=\"$guppyThemeURL/help.gif\"title=\"$fieldSpec[hint]\" align=\"middle\"/>\n";
  }

  guppy_endTableCell($element, array());

  return true;
}


function guppy_buildField($componentSpec, $element, $data, $rowIndex, $inTable)
{
  $componentName = $componentSpec['name'];

    // Check for existence of field name and get it
  if (!isset($element['name']))
    return guppy_generateError("$componentName"."[input element]", "missing element name");
  $fieldName = $element['name'];

    // Check for existence of field spec in component spec and get it
  if (!isset($componentSpec['fields'][$fieldName]))
    return guppy_generateError("$componentName", "undefined field '$fieldName'");
  $fieldSpec = $componentSpec['fields'][$fieldName];
  $fieldType = $fieldSpec['type'];

    // Get field hint and set it as html title
  $fieldHint = isset($element['hint'])
               ? $element['hint']
               : (isset($fieldSpec['hint']) ? $fieldSpec['hint'] : null);
  if ($fieldHint)
    $htmlTitle = " title=\"" . guppy_translate($fieldHint) . "\"";

    // Get read-only info merged from spec and layout
  $readonly = guppy_getMergedBoolean($fieldSpec, $element, 'readonly', false);

    // Get mandatory info merged from spec and layout
  $mandatory = guppy_getMergedBoolean($fieldSpec, $element, 'mandatory', false);

  $formFieldName = "f:$componentName:$fieldName:$rowIndex";
  $fieldData = $data[$fieldName];

  guppy_startTableCell($element, $fieldSpec, $inTable);

  $style = guppy_getSizeStyle($element);

  if ($fieldType == 'select')
  {
    $options = $fieldSpec['options'];

    echo "<select name=\"$formFieldName\"$style$htmlTitle>\n";

    $i = 0;
    foreach ($options as $option)
    {
      $value = $options[$i]['value'];
      $selected = ($value === $fieldData ? " selected=\"1\"" : "");
      //echo "$formFieldName: ($value/$fieldData)(" . gettype($value) . "/" . gettype($fieldData) . ")<br>\n";
      $optionTitle = guppy_translate($option[title]);
      echo "<option value=\"$i\"$selected>" . htmlspecialchars($optionTitle) . "</option>\n";
      ++$i;
    }
    echo "</select>\n";
  }
  else if ($fieldType == 'bool')
  {
    $htmlClass = "b";
    if ($readonly)
    {
      $readonlyHTML = " disabled=\"1\"";
      $htmlClass .= " ro";
    }

    if ($fieldData)
      $checked = " checked=\"1\"";

    echo "<input name=\"$formFieldName\" type=\"checkbox\" class=\"$htmlClass\" value=\"1\"$readonlyHTML$checked$htmlTitle/>\n";
  }
  else if ($fieldType == 'radio')
  {
    $formFieldName = "f:$componentName:$fieldName";
    $htmlClass = "o";
    if ($readonly)
    {
      $readonlyHTML = " readonly=\"1\"";
      $htmlClass .= " ro";
    }

    if ($fieldData)
      $checked = " checked";

    echo "<input name=\"$formFieldName\" type=\"radio\" class=\"$htmlClass\" value=\"$rowIndex\"$readonlyHTML$checked$htmlTitle/>\n";
  }
  else
  {
    switch ($fieldType)
    {
      case 'int':
        $htmlType = 'text';
        $htmlClass = 'i';
      break;

      case 'real':
        $htmlType = 'text';
        $htmlClass = 'r';
      break;

      case 'date':
        $htmlType = 'text';
        $htmlClass = 'd';
      break;

      case 'time':
        $htmlType = 'text';
        $htmlClass = 'd';
      break;

      case 'datetime':
        $htmlType = 'text';
        $htmlClass = 'dt';
        $fieldData = strftime("%b %d %Y %H:%M:%S", $fieldData);
      break;

      case 'string':
        $htmlType = 'text';
        $htmlClass = 's';
      break;

      case 'image':
        $htmlType = 'text';
        $htmlClass = 'm';
      break;

      default:
        return guppy_generateError("$componentName", "unknown field type '$fieldType' for field '$fieldName'");
    }

    if ($element['view'] == 'html')
    {
      echo "<div class=\"htmlarea\">\n";
      echo "\n<textarea id=\"$formFieldName\" name=\"$formFieldName\"$style>" . htmlspecialchars($fieldData) . "</textarea>\n";
      echo "</div>\n";

      $configHeight = isset($element['height']) ? $element['height'] : 200;
      $configWidth  = isset($element['width'])  ? $element['width']  : 300;
      $postnukeThemeURL = guppy_getSetting('htmlAreaStyled') ? 'postnukeThemeURL' : 'null';
      $enableUndo = (guppy_getSetting('htmlAreaUndo') ? 'true' : 'false');
      $enableWordKill = (guppy_getSetting('htmlAreaWordKill') ? 'true' : 'false');

      if (guppy_getSetting('htmlAreaEnabled'))
      {
        echo  "<script type=\"text/javascript\" defer=\"1\">\n"
            . "guppyHtmlAreaInit('$formFieldName', $postnukeThemeURL, $configWidth, $configHeight, "
            . "$enableUndo, $enableWordKill);\n</script>\n";
      }
    }
    else 
    if ($element['view'] == 'text')
    {
      echo "\n<textarea name=\"$formFieldName\"$style>" . htmlspecialchars($fieldData) . "</textarea>\n";
    }
    else
    {
      if ($readonly)
      {
        $readonlyHTML = " readonly=\"1\"";
        $htmlClass .= " ro";
      }

      if ($mandatory)
        if ($fieldData == '')
          $htmlClass .= " mde";
        else
          $htmlClass .= " mdt";

      echo "<input name=\"$formFieldName\" id=\"$formFieldName\" type=\"$htmlType\" class=\"$htmlClass\" value=\""
           . htmlspecialchars($fieldData). "\"$style$readonlyHTML$htmlTitle"
           ." onKeyUp=\"handleOnKeyUpInput(this,'$mandatory')\"/>";

      if ($fieldType == 'date'  &&  !$readonly)
      {
        echo "<button type=\"button\" id=\"$formFieldName:db\">...</button>\n";

        echo   "<script type=\"text/javascript\">\nCalendar.setup({"
             . "inputField: \"$formFieldName\", "
             . "ifFormat: \"y-m-d\", "
             . "button: \"$formFieldName:db\", "
             . "singleClick: true})\n"
             . "</script>\n";
      }
      else if ($fieldType == 'image'  &&  !$readonly)
      {
        $photoshareFindImageURL = pnModUrl('photoshare','user','findimage', 
                                           array('url' => 'relative', 'html' => 'url'));
        $photoshareThumbnailSize = pnModGetVar('photoshare', 'thumbnailsize');

        echo "<button type=\"button\" "
             . "onClick=\"guppyFindImage('$formFieldName','$photoshareFindImageURL','$photoshareThumbnailSize')\">...</button>\n";
      }
    }
  }

  guppy_endTableCell($element, array());

  return true;
}


function guppy_buildAction($componentSpec, $element, $data, $rowIndex)
{
  $componentName = $componentSpec['name'];

    // Check for existence of action name and get it
  if (!isset($element['name']))
    return guppy_generateError("$componentName"."[action element]", "missing element name");
  $actionName = $element['name'];

    // Check for existence of action spec in component spec and get it
  if (!isset($componentSpec['actions'][$actionName]))
    return guppy_generateError("$componentName", "undefined action '$actionName'");
  $actionSpec = $componentSpec['actions'][$actionName];

    // Get optional action title (first from layout then from action spec)
  if (isset($element['title']))
    $actionTitle = $element['title'];
  else if (isset($actionSpec['title']))
    $actionTitle = $actionSpec['title'];
  else
    $actionTitle = NULL;

  if ($actionTitle == NULL)
    return guppy_generateError("$componentName", "missing action title for '$actionName'");

  $actionTitle = guppy_translate($actionTitle);

  guppy_startTableCell($element, array(), false);

  echo "<span onClick=\"handlOnClickAction('$actionName','$componentName','$rowIndex')\" class=\"clickable action\">"
       . htmlspecialchars($actionTitle). "</span>";

  guppy_endTableCell($element, array());
}


function guppy_buildRaw($componentSpec, $element, $data, $rowIndex)
{
  $componentName = $componentSpec['name'];

    // Check for existence of field name and get it
  if (!isset($element['name']))
    return guppy_generateError("$componentName"."[raw element]", "missing element name");
  $fieldName = $element['name'];

    // Check for existence of field spec in component spec and get it
  if (!isset($componentSpec['fields'][$fieldName]))
    return guppy_generateError("$componentName", "undefined field '$fieldName'");
  $fieldSpec = $componentSpec['fields'][$fieldName];

  $fieldData = $data[$fieldName];

  guppy_startTableCell($element, array(), false);

  echo "$fieldData";

  guppy_endTableCell($element, array());
}


/*========================================================================
  Output caching
========================================================================*/

function guppy_beginOutput()
{
  ob_start();
}


function guppy_output()
{
  global $guppy_outputCleared;
  if ($guppy_outputCleared)
    return true;

  $output = ob_get_contents();
  ob_end_clean();
  return $output;
}


/*========================================================================
  Window cache
========================================================================*/

function guppy_getNewWindowKey()
{
  srand((double)microtime()*1000000);
  return rand();
}


/* DEBUGGING
function guppy_dumpcache($windowCache)
{
  for ($i=0; $i<count($windowCache['cache']); ++$i)
  { 
    $c = $windowCache['cache'][$i];
    if ($c != null)
    {
      guppylog("Cache($i) - $c[createdTime] (key $c[windowKey]):"); 
      $keys = array_keys($c['window']['spec']['components']);
      foreach ($keys as $k)
        guppylog("    " . $k);
    }
    else
      guppylog("Cache($i) empty.");
  }
}
//*/

function guppy_getNewWindowID()
{
  $windowCache = pnSessionGetVar('guppyWindowCache');
  if (!isset($windowCache))
    pnSessionSetVar('guppyWindowCache', array( 'count' => 0, 'cache' => array()));
  
  $windowCache = pnSessionGetVar('guppyWindowCache');
  
  //guppylog("\nGetNewWindowID.\nCount = " . count($windowCache['cache']));
  //guppy_dumpcache($windowCache);

  if (count($windowCache['cache']) >= 5)
    $windowIndex = guppy_cleanupWindowId($windowCache);
  else
    $windowIndex = count($windowCache['cache']);

  $windowCache['cache'][$windowIndex] = array( 'createdTime' => $windowCache['count'],
                                               'window'      => array() );
  ++$windowCache['count'];

  pnSessionSetVar('guppyWindowCache', $windowCache);
  
  //guppy_dumpcache($windowCache);

  return $windowIndex;
}


function guppy_setWindowData($windowID, $windowKey, $windowData)
{
  //guppylog("\nSet WindowData for ID $windowID (key: $windowKey).");

  $windowCache = pnSessionGetVar('guppyWindowCache');
  $windowCache['cache'][$windowID]['window']    = $windowData;
  $windowCache['cache'][$windowID]['windowKey'] = $windowKey;
  
  pnSessionSetVar('guppyWindowCache', $windowCache);

  //$windowCache = pnSessionGetVar('guppyWindowCache'); // DEBUG
  //guppy_dumpcache($windowCache);
}


function guppy_getWindowData($windowID, &$windowKey)
{
  $windowCache = pnSessionGetVar('guppyWindowCache');
  if (!isset($windowCache))
    return false;

  //guppylog("\nGet WD $windowID.");

  $windowCache = pnSessionGetVar('guppyWindowCache');

  $windowKey = $windowCache['cache'][$windowID]['windowKey'];

  //guppy_dumpcache($windowCache);
  //guppylog("=> gave key $windowKey.");

  return $windowCache['cache'][$windowID]['window'];
}


function guppy_deleteWindowID($windowID)
{
  $windowCache = pnSessionGetVar('guppyWindowCache');

  //guppylog("\n* DeleteID $windowID - start.\nCount: " . count($windowCache['cache']));
  //guppy_dumpcache($windowCache);

  $windowCache['cache'][$windowID] = null;
  
  //guppylog("\nDeleteID $windowID.\nCount: " . count($windowCache['cache']));
  //guppy_dumpcache($windowCache);

  pnSessionSetVar('guppyWindowCache', $windowCache);
}


function guppy_cleanupWindowId(&$windowCache)
{
  //guppylog("\nStart CleanupWindowId.");
  //guppy_dumpcache($windowCache);

  $oldestWindowIndex = 0;

  for ($i=count($windowCache['cache'])-1; $i>=0 ; --$i)
    if ($windowCache['cache'][$i] == null)
    {
      $oldestWindowIndex = $i;
      break;
    }
    else if ($windowCache['cache'][$i]['createdTime'] < $windowCache['cache'][$oldestWindowIndex]['createdTime'])
      $oldestWindowIndex = $i;

  $windowCache['cache'][$oldestWindowIndex] = null;

  //guppylog("\nEnd CleanupWindowId.\nOldest ID: $oldestWindowIndex");
  //guppy_dumpcache($windowCache);

  return $oldestWindowIndex;
}


/*========================================================================
  Helpers
========================================================================*/

  // Load file content into string and return it
function guppy_loadFile($funcName, $filename)
{
  $fd = fopen("$filename", "rb");
  if ($fd === false)
    return guppy_generateError($funcName, "Unable to load file '$filename'");
  $content = fread($fd, filesize($filename));
  fclose($fd);

  return $content;
}


function guppy_getMergedBoolean($spec, $layout, $name, $defaultValue)
{
  if (!isset($spec[$name])  &&  !isset($layout[$name]))
    return $defaultValue;

  if (!isset($spec[$name])  &&  isset($layout[$name]))
    return $layout[$name];

  if (isset($spec[$name])  &&  !isset($layout[$name]))
    return $spec[$name];

  return $spec[$name] || $layout[$name];
}


function guppy_getSizeStyle(&$element)
{
  $style = "";

  if (isset($element['width']))
    $style .= "width: $element[width]px;";
  if (isset($element['height']))
    $style .= "height: $element[height]px;";

  if ($style != '')
    $style = " style=\"$style\"";

  return $style;
}


function guppy_getElementID($name = null)
{
  static $count = 0;
  static $idMap = array();

  if ($name == null)
  {
    return "elem" . ++$count;
  }
  else
  {
    if (isset($idMap[$name]))
      return $idMap[$name];

    $idMap[$name] = "elem" . ++$count;

    return $idMap[$name];
  }
}

?>
