<?php
// $Id: pnuserapi.php,v 1.87 2004/07/01 16:54:19 jornlind Exp $
// =======================================================================
// Pagesetter by Jorn Lind-Nielsen (C) 2003-2004.
// ----------------------------------------------------------------------
// For POST-NUKE Content Management System
// Copyright (C) 2002 by the PostNuke Development Team.
// http://www.postnuke.com/
// ----------------------------------------------------------------------
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WithOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html
// =======================================================================

require_once("modules/pagesetter/common.php");


function pagesetter_userapi_getPubList($args)
{
  if (!isset($args['tid'])  ||  $args['tid']=='')
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'tid' in 'pagesetter_userapi_getPubList'", false);

    // (using null values here for no particular reason)
  $tid               = $args['tid'];
  $orderBy           = isset($args['orderBy']) ? $args['orderBy'] : null;
  $orderByStr        = !empty($args['orderByStr']) ? $args['orderByStr'] : null;
  $topic             = isset($args['topic']) ? $args['topic'] : -1;
  $useRestrictions   = isset($args['useRestrictions']) ? $args['useRestrictions'] : true;
  $hideDepot         = isset($args['hideDepot']) ? $args['hideDepot'] : true;
  $showDeleted       = isset($args['showDeleted']) ? $args['showDeleted'] : false;
  $getTextual        = isset($args['getTextual']) ? $args['getTextual'] : true;
  $language          = isset($args['language']) ? $args['language'] : null;
  $filter            = isset($args['filter']) ? $args['filter'] : null; // Field names in API notation!
  $filterStrSet      = isset($args['filterSet']) ? $args['filterSet'] : null; // Field names in user notation
  $noOfItems         = isset($args['noOfItems']) ? $args['noOfItems'] : null; // using null here makes zero items possible
  $offsetItems       = isset($args['offsetItems']) ? $args['offsetItems'] : null;
  $offsetPage        = isset($args['offsetPage']) ? $args['offsetPage'] : null;
  $getOwners         = isset($args['getOwners']) ? $args['getOwners'] : null;
  
    // Check access
  if (!pnSecAuthAction(0, 'pagesetter::', "$tid::", ACCESS_READ))
    return pagesetterErrorApi(__FILE__, __LINE__, _PGNOAUTH);

  if (!pnModAPILoad('pagesetter', 'admin'))
    return pagesetterErrorApi(__FILE__, __LINE__, 'Failed to load Pagesetter admin API');

    // Get publication info for title field name

  $pubInfo =  pnModAPIFunc( 'pagesetter',
                            'admin',
                            'getPubTypeInfo',
                            array('tid' => $tid) );

  if ($pubInfo === false)
    return false;

    // Get default number of items of none is specified
  if ($noOfItems === null)
    $noOfItems = $pubInfo['publication']['listCount'];

    // Calculate item offset, if not specified, based on current page
  if ($offsetItems == null)
  {
    if ($offsetPage == null)
      $offsetItems = 0;
    else
      $offsetItems = $offsetPage * $noOfItems;
  }

    // Get filter set expression
  if ($filterStrSet == null)
    $filterSetSQL = null;
  else
    $filterSetSQL = pnModAPIFunc( 'pagesetter',
                                  'user',
                                  'parseFilter',
                                  array('tid'    => $tid,
                                        'filter' => $filterStrSet) );

  if ($filterSetSQL === false)
    return false;


    // Get default ordering if not in args
  if ($orderBy === null  &&  $orderByStr === null)
  {
    $orderBy = array();

    if ($pubInfo['publication']['sortField1'] != NULL)
      $orderBy[] = array( 'name' => $pubInfo['publication']['sortField1'],
                          'desc' => $pubInfo['publication']['sortDesc1'] );
    if ($pubInfo['publication']['sortField2'] != NULL)
      $orderBy[] = array( 'name' => $pubInfo['publication']['sortField2'],
                          'desc' => $pubInfo['publication']['sortDesc2'] );
    if ($pubInfo['publication']['sortField3'] != NULL)
      $orderBy[] = array( 'name' => $pubInfo['publication']['sortField3'],
                          'desc' => $pubInfo['publication']['sortDesc3'] );
  }
  else if ($orderByStr !== null)
  {
      // Get order by string expression
    $orderBy = pnModAPIFunc( 'pagesetter',
                             'user',
                             'parseOrderBy',
                             array('tid'        => $tid,
                                   'orderByStr' => $orderByStr) );

    if ($orderBy === false)
      return false;
  }

  $titleField = pagesetterGetPubColumnName($pubInfo['publication']['titleFieldID']);

    // Make database query

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $pubTable        = pagesetterGetPubTableName($tid);
  $pubColumn       = $pntable['pagesetter_pubdata_column'];
  $pubHeaderTable  = $pntable['pagesetter_pubheader'];
  $pubHeaderColumn = $pntable['pagesetter_pubheader_column'];
  $topicsTable     = $pntable['topics'];
  $topicsColumn    = $pntable['topics_column'];

  $sql = "SELECT $pubTable.$pubColumn[id],
                 $pubTable.$pubColumn[pid],
                 $pubColumn[author],
                 UNIX_TIMESTAMP($pubColumn[created]) as $pubColumn[created],
                 $pubColumn[approvalState],
                 $topicsColumn[topicname],
                 $pubColumn[revision],
                 $pubColumn[online],
                 $titleField
          FROM   $pubTable 
                 LEFT JOIN $topicsTable ON $pubColumn[topic] = $topicsColumn[topicid]
                 LEFT JOIN $pubHeaderTable ON $pubHeaderColumn[tid] = '" . pnVarPrepForStore($tid) . "' AND
                                              $pubHeaderTable.$pubHeaderColumn[pid] = $pubTable.$pubColumn[pid]";

  if ($filterSetSQL != null)
    $sql .= $filterSetSQL['join'];

  $whereSQL = '';

    // Add publish restrictions

  if ($useRestrictions)
  {
    if ($language != null)
      $languageRestriction = "\nAND ($pubColumn[language] = '" . pnVarPrepForStore($language) . "' OR $pubColumn[language] = 'x_all')";
    else
      $languageRestriction = "";

    $whereSQL .= "\nWHERE ($pubColumn[publishDate] <= CURDATE() || $pubColumn[publishDate] IS NULL)"
                 . "\nAND  (CURDATE() < $pubColumn[expireDate] || $pubColumn[expireDate] IS NULL)"
                 . "\nAND $pubColumn[online]"
                 . "\nAND $pubColumn[showInList]"
                 . "$languageRestriction";

    if ($topic >= 0)
      $whereSQL .= "\nAND $pubColumn[topic] = " . pnVarPrepForStore($topic);
  }

    // Add filter restrictions
    // These filters are a bit simpler than the filter set and always uses "LIKE" operator.
    // Used solely for filtering in the editor's publication list

  if ($filter != null)
  {
    $first = true;

    foreach ($filter as $field => $value)
    {
      if (isset($value)  &&  $value !== '')
      {
        if ($first)
        {
          if ($whereSQL == '')
            $whereSQL = "\nWHERE";
          $first = false;
        }
        else
        {
          $whereSQL .= " AND";
        }

        if ($field == 'author')
          $whereSQL .= " " . $pubColumn[$field] . " LIKE '%" . pnVarPrepForStore($value) . "%'";
        else if ($field == 'title')
          $whereSQL .= " " . $titleField . " LIKE '%" . pnVarPrepForStore($value) . "%'";
        else
          $whereSQL .= " " . $pubColumn[$field] . " = '" . pnVarPrepForStore($value) . "'";
      }
    }
  }

  if ($filterSetSQL != null)
  {
    if ($whereSQL == '')
      $whereSQL = "\nWHERE (";
    else
      $whereSQL .= "\nAND (";

    $whereSQL .= $filterSetSQL['sql'] . ")";
  }

  if ($hideDepot)
  {
    if ($whereSQL == '')
      $whereSQL .= "\nWHERE $pubColumn[inDepot] = 0";
    else
      $whereSQL .= "\nAND NOT $pubColumn[inDepot]";
  }

  if ($getOwners)
  {
    if ($whereSQL == '')
      $whereSQL .= "\nWHERE $pubColumn[creatorID] = " . pnUserGetVar('uid');
    else
      $whereSQL .= "\nAND $pubColumn[creatorID] = " . pnUserGetVar('uid');
  }

  $sql .= $whereSQL;

  if ($showDeleted)
    $sql .= "\nGROUP BY $pubTable.$pubColumn[pid]";

    // Add ordering

  if (count($orderBy) > 0)
  {
    $first = true;
    $sql .= "\nORDER BY ";
    foreach ($orderBy as $field)
    {
      if (!$first)
        $sql .= ", ";
      $first = false;

        // assume name is stored as SQL column name
      if ($field['name'] == $pubColumn['topic']) // Don't sort by topic ID - sort by name
        $sql .= $topicsColumn[topicname];
      else
        $sql .= $field['name']; 
      if ($field['desc'])
        $sql .= " DESC";
    }

    $sql .= ", $pubColumn[revision]";
  }
  else
    $sql .= "\nORDER BY $pubColumn[revision]";

  //echo "<pre>$sql</pre>"; exit(0);

  if ($noOfItems > 0  ||  $offsetItems > 0)
      // Select one extra to see if more is available
    $result = $dbconn->selectLimit($sql, $noOfItems+1, $offsetItems);
  else
    $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return pagesetterErrorApi(__FILE__, __LINE__, '"getPubList" failed: ' 
                                                  . $dbconn->errorMsg() . " while executing: <pre>$sql</pre>");   

    // Get workflow information in order to show state titles

  if (!pnModAPILoad('pagesetter', 'workflow'))
    return pagesetterErrorAPI(__FILE__, __LINE__, 'Failed to load Pagesetter workflow API');

  $workflowName = $pubInfo['publication']['workflow'];
  $workflow = pnModAPIFunc('pagesetter', 'workflow', 'load', array('workflow' => $workflowName) );
  if ($workflow === false)
    return false;


  $pubList = array();
  $moreItems = false;

  for ($cou=1; !$result->EOF; $result->MoveNext(), ++$cou)
  {
    $id  = $result->fields[0];
    $pid = $result->fields[1];
    
    if (pnSecAuthAction(0, 'pagesetter::', "$tid:$pid:", ACCESS_READ))
    {
      if ($noOfItems > 0  &&  $cou == $noOfItems+1)
      {
        $moreItems = true;
      }
      else
      {
        if ($getTextual)
          $approval = pagesetterApprovalState2String($result->fields[4], $workflow);
        else
          $approval = $result->fields[4];

        $pubList[] = array('id'             => $id,
                           'pid'            => $pid,
                           'author'         => $result->fields[2],
                           'created'        => $result->fields[3],
                           'approvalState'  => $approval,
                           'topic'          => $result->fields[5],
                           'revision'       => $result->fields[6],
                           'online'         => $result->fields[7],
                           'title'          => $result->fields[8]);
      }
    }
  }

  $result->Close();

  return array( 'publications' => &$pubList,
                'more'         => $moreItems );
}


/* Calling conventions:
 *  If $id is set then we want a specific version.
 *  - $id must always be set when using the administrative interface.
 *  If not $id is set then we want the current online version, so $pid must be set.
 * - this is when the publication is viewed as standard, and so restrictions should always be used.
 */
function pagesetter_userapi_getPub($args)
{
  if (!isset($args['tid'])  ||  $args['tid']=='')
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'tid' in 'pagesetter_userapi_getPub'");
  if (!isset($args['pid'])  &&  !isset($args['id']))
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'pid' or 'id' in 'pagesetter_userapi_getPub'");
  if (isset($args['pid'])  &&  isset($args['id']))
    return pagesetterErrorApi(__FILE__, __LINE__, "Cannot use arguments 'pid' and 'id' simultaneous in 'pagesetter_userapi_getPub'");

  $tid             = $args['tid'];
  $pid             = (isset($args['pid']) ? $args['pid'] : null);
  $id              = (isset($args['id']) ? $args['id'] : null);
  $format          = isset($args['format']) ? $args['format'] : 'database';

    // Get publication type info (also used for access control)
  
  $pubInfo =  pnModAPIFunc( 'pagesetter',
                            'admin',
                            'getPubTypeInfo',
                            array('tid' => $tid) );

  if ($pubInfo === false)
    return false;

  $useRestrictions = true;
  $getOwnOnly = false;

  if ($id == null)
  {
    if (!pnSecAuthAction(0, 'pagesetter::', "$tid:$pid:", ACCESS_READ))
      return pagesetterErrorApi(__FILE__, __LINE__, _PGNOAUTH);
  }
  else
  {
    if (!pnSecAuthAction(0, 'pagesetter::', "$tid::", pagesetterAccessAuthor))
      return pagesetterErrorApi(__FILE__, __LINE__, _PGNOAUTH);
    
    if ($pubInfo['publication']['enableEditOwn'])
    {
      if (!pnSecAuthAction(0, 'pagesetter::', "$tid::", pagesetterAccessEditor))
        $getOwnOnly = true;
    }
    else
    {
      if (!pnSecAuthAction(0, 'pagesetter::', "$tid::", pagesetterAccessEditor))
        return pagesetterErrorApi(__FILE__, __LINE__, _PGNOAUTH);
    }
  
    $useRestrictions = false;
  }

  if (!pnModAPILoad('pagesetter', 'admin'))
    return pagesetterErrorApi(__FILE__, __LINE__, 'Failed to load Pagesetter admin API');

    // Make database query

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $pubTable        = pagesetterGetPubTableName($tid);
  $pubColumn       = $pntable['pagesetter_pubdata_column'];
  $pubHeaderTable  = $pntable['pagesetter_pubheader'];
  $pubHeaderColumn = $pntable['pagesetter_pubheader_column'];
  $listItemsTable  = $pntable['pagesetter_listitems'];
  $topicsTable     = $pntable['topics'];
  $topicsColumn    = $pntable['topics_column'];

  $fieldSQL = "";
  $joinSQL = "";
  foreach ($pubInfo['fields'] as $field)
  {
    $fieldID    = $field['id'];
    $columnName = pagesetterGetPubColumnName($fieldID);

    if ($field['type'] >= pagesetterFieldTypeListoffset)
    {
        // Add extra fields for title, fullTitle, and value of list item
      $fieldSQL .=   ",\n{$columnName}_table.pg_title as {$columnName}_title,\n"
                   . "{$columnName}_table.pg_fullTitle as {$columnName}_fullTitle,\n"
                   . "{$columnName}_table.pg_description as {$columnName}_description,\n"
                   . "{$columnName}_table.pg_value as {$columnName}_value";

        // and add extra JOIN statement
      $joinSQL .=   "\nLEFT JOIN $listItemsTable as {$columnName}_table\n"
                  .   "       ON {$columnName}_table.pg_id = $columnName";
    }

    $fieldSQL .= ",\n" . $columnName;
  }

  if ($id != null)
    $pubKeySQL = "pubTable.$pubColumn[id] = " . pnVarPrepForStore($id);
  else
    $pubKeySQL = "pubTable.$pubColumn[pid] = " . pnVarPrepForStore($pid);

  $sql = "SELECT pubTable.$pubColumn[id],
                 pubTable.$pubColumn[pid],
                 pubTable.$pubColumn[approvalState],
                 pubTable.$pubColumn[online],
                 pubTable.$pubColumn[revision],
                 pubTable.$pubColumn[topic],
                 $topicsColumn[topicname] as core_topic_name,
                 $topicsColumn[topictext] as core_topic_text,
                 $topicsColumn[topicimage] as core_topic_image,
                 pubTable.$pubColumn[showInMenu],
                 pubTable.$pubColumn[showInList],
                 pubTable.$pubColumn[author],
                 pubTable.$pubColumn[creatorID],
                 UNIX_TIMESTAMP(pubTable.$pubColumn[created]) as $pubColumn[created],
                 $pubHeaderTable.$pubHeaderColumn[hitCount],
                 UNIX_TIMESTAMP(pubTable.$pubColumn[lastUpdated]) as $pubColumn[lastUpdated],
                 pubTable.$pubColumn[publishDate],
                 pubTable.$pubColumn[expireDate],
                 pubTable.$pubColumn[language]
                 $fieldSQL
          FROM   $pubTable as pubTable
          $joinSQL
          LEFT JOIN $topicsTable ON pubTable.$pubColumn[topic] = $topicsColumn[topicid]
          LEFT JOIN $pubHeaderTable ON pubTable.$pubColumn[pid] = $pubHeaderTable.$pubHeaderColumn[pid] AND
                                       $pubHeaderColumn[tid] = " . pnVarPrepForStore($tid) . "
          WHERE  $pubKeySQL";

  if ($useRestrictions)
  {
    $sql .=   "\nAND ($pubColumn[publishDate] <= CURDATE() || $pubColumn[publishDate] IS NULL)"
            . "AND  (CURDATE() < $pubColumn[expireDate] || $pubColumn[expireDate] IS NULL)"
            . "\nAND $pubColumn[online]"
            . "\nAND NOT $pubColumn[inDepot]";
  }

  if ($getOwnOnly)
  {
    $sql .= "\nAND $pubColumn[creatorID] = " . pnUserGetVar('uid');
  }

  //echo "<pre>$sql</pre>"; exit(0);

  global $ADODB_FETCH_MODE;
  $oldMode = $ADODB_FETCH_MODE;
  $ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
  $result = $dbconn->execute($sql);
  $ADODB_FETCH_MODE = $oldMode;

  if ($dbconn->errorNo() != 0)
    return pagesetterErrorApi(__FILE__, __LINE__, '"getPub" failed: ' 
                                                  . $dbconn->errorMsg() . " while executing: <pre>$sql</pre>");   

  if ($result->EOF)
    return true;

    // fetch title - perhaps from list item
  //$title = pagesetterConvertValueToListItemTitle($pubInfo, $result, $pubInfo['publication']['titleFieldID']);

  if ($format == 'user')
    $topic = array( 'id'    => intval($result->fields[$pubColumn['topic']]),
                    'name'  => $result->fields['core_topic_name'],
                    'text'  => $result->fields['core_topic_text'],
                    'image' => $result->fields['core_topic_image'] );
  else
    $topic = intval($result->fields[$pubColumn['topic']]);

  if (!pnModAPILoad('pagesetter', 'workflow'))
    return pagesetterErrorApi(__FILE__, __LINE__, 'Failed to load Pagesetter workflow API');

  $workflowName = $pubInfo['publication']['workflow'];
  $workflow = pnModAPIFunc('pagesetter', 'workflow', 'load', array('workflow' => $workflowName) );
  if ($workflow === false)
    return false;
  
  $approvalState = $result->fields[$pubColumn['approvalState']];
  $approvalTitle = pagesetterApprovalState2String($approvalState, $workflow);

  $pubCore = array( 'id' => $result->fields[$pubColumn['id']],
                    'pid' => $result->fields[$pubColumn['pid']],
                    'tid' => $tid,
                    'approvalState' => $approvalState,
                    'approvalTitle' => $approvalTitle,
                    'online' => intval($result->fields[$pubColumn['online']]),
                    'revision' => intval($result->fields[$pubColumn['revision']]),
                    'topic' => $topic,
                    'showInMenu' => intval($result->fields[$pubColumn['showInMenu']]),
                    'showInList' => intval($result->fields[$pubColumn['showInList']]),
                    'author' => $result->fields[$pubColumn['author']],
                    'creatorID' => $result->fields[$pubColumn['creatorID']],
                    'creator' => pnUserGetVar('uname',$result->fields[$pubColumn['creatorID']]),
                    'created' => $result->fields[$pubColumn['created']],
                    'hitCount' => intval($result->fields[$pubHeaderColumn['hitCount']]),
                    'lastUpdated' => $result->fields[$pubColumn['lastUpdated']],
                    'publishDate' => $result->fields[$pubColumn['publishDate']],
                    'expireDate' => $result->fields[$pubColumn['expireDate']],
                    'language' => $result->fields[$pubColumn['language']],
                    'title' => $result->fields[pagesetterGetPubColumnName($pubInfo['publication']['titleFieldID'])]  );

  if ($format != 'user')
  {
      // Convert all core field names "X" to "coreX"
    pagesetterFlattenPubData($pubCore);
  }

  $pid = $pubCore['pid'];

  if ($format == 'user')
    $pub = array('core' => $pubCore);
  else
    $pub = &$pubCore;

  foreach ($pubInfo['fields'] as $field)
  {
    $fieldID    = $field['id'];
    $columnName = pagesetterGetPubColumnName($fieldID);
    $fieldName = $field['name'];

    if ($fieldID == $pubInfo['publication']['pageableFieldID']  &&  $format == 'user')
    {
        // Split multi-page field by <hr class="pagebreak"/> tag
      $pages = pagesetterSplitPages($result->fields[$columnName]);
      $pub[$fieldName] = $pages;

        // Add page count to core
      $pub['core']['pageCount'] = count($pages);
    }
    else if ($field['type'] >= pagesetterFieldTypeListoffset)
    {
        // Add various list properties to list field
      if ($format == 'user')
        $pub[$fieldName] = array( 'id'          => intval($result->fields[$columnName]),
                                  'value'       => $result->fields[$columnName . '_value'],
                                  'title'       => $result->fields[$columnName . '_title'],
                                  'description' => $result->fields[$columnName . '_description'],
                                  'fullTitle'   => $result->fields[$columnName . '_fullTitle'] );
      else
        $pub[$fieldName] = intval($result->fields[$columnName]);
    }
    else
      $pub[$fieldName] = $result->fields[$columnName];
  }

  $result->Close();

  return $pub;
}


function pagesetter_userapi_getPubFormatted($args)
{
  if (!isset($args['tid'])  ||  $args['tid']=='')
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'tid' in 'pagesetter_userapi_getPubFormatted'");
  if (!isset($args['pid'])  &&  !isset($args['id']))
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'pid' in 'pagesetter_userapi_getPubFormatted'");

  $tid             = $args['tid'];
  $pid             = $args['pid'];
  $id              = $args['id'];
  $format          = (isset($args['format']) ? $args['format'] : 'full');
  $coreExtra       = (isset($args['coreExtra']) ? $args['coreExtra'] : array());
  $updateHitCount  = (isset($args['updateHitCount']) ? $args['updateHitCount'] : true);

    // Access check depends on how the publication is viewed - see later

    // Check for cached version

  $smarty = new pnRender('pagesetter'); // Used in blocks, so supply module name.

    // Ignore caching when fetching specific version(id)
  if (isset($id))
    pagesetterSmartyClearCache($tid, $pid);

    // Some of the extra core elements influence caching!
  $cacheID = pagesetterGetPublicationUniqueID($tid, $pid, pnUserGetLang(), $coreExtra['page']);
  $templateFile = pagesetterSmartyGetTemplateFilename($smarty, $tid, $format);

  if (!$smarty->is_cached($templateFile, $cacheID))
  {
      // No cache - generate page
      // Permission control in API getPub()

    if (!pnModAPILoad('pagesetter', 'admin'))
      return pagesetterErrorApi(__FILE__, __LINE__, 'Failed to load Pagesetter admin API');


      // Get publication data and type info
    
    $pub = pnModAPIFunc( 'pagesetter',
                         'user',
                         'getPub',
                         array('tid'    => $tid,
                               'pid'    => $pid,
                               'id'     => $id,
                               'format' => 'user') );

    if ($pub === false)
      return false;
    if ($pub === true)
      return pagesetterErrorApi(__FILE__, __LINE__, _PGUNKNOWNPUB);

    $pubInfo =  pnModAPIFunc( 'pagesetter', 'admin', 'getPubTypeInfo',
                              array('tid' => $tid) );

    if ($pubInfo === false)
      return false;

      // Escape necessary values
    foreach ($pub as $key => $value)
      if (is_string($value))
        $pub[$key] = pnVarPrepHTMLDisplay($value);

      // Add template variable for "print this" and such like

    $printThisURL = pnModUrl('pagesetter','user','printpub',
                             array('tid'    => $tid,
                                   'pid'    => $pid));
    $printThisHTML = "<a href=\"$printThisURL\">" . _PGPRINTTHIS . "</a>";
    $pub['core']['printThisURL'] = $printThisURL;
    $pub['core']['printThis'] = $printThisHTML;

    $sendThisURL = pnModUrl('pagesetter','user','sendpub',
                            array('tid'    => $tid,
                                  'pid'    => $pid));
    $sendThisHTML = "<a href=\"$sendThisURL\">" . _PGSENDTHIS . "</a>";
    $pub['core']['sendThisURL'] = $sendThisURL;
    $pub['core']['sendThis'] = $sendThisHTML;


    $fullURL = pnModUrl('pagesetter','user','viewpub',
                        array('tid'    => $tid,
                              'pid'    => $pid));
    $pub['core']['fullURL'] = $fullURL;


      // Add additional core values from caller
    $pub['core'] += $coreExtra;

      // Create page from data and type info

    $smarty->assign($pub);

    if (!$smarty->template_exists($templateFile))
      return _PGSORRYNOTEMPLATE . ": $templateFile<p>";

    $hitCount = $pub['core']['hitCount'];

      // May perhaps not be specified on URL
    $pid = $pub['core']['pid'];
    $id  = $pub['core']['id'];

      // Ensure cache expires when the publication expires
    if (isset($pub['core']['expireDate']))
    {
      list($year,$month,$day) = split('-',$pub['core']['expireDate']);
      $expireTime = mktime(0,0,0,$month,$day,$year);
      $smarty->cache_lifetime =  $expireTime - time();
    }
  }
  else
  {
      // For the cached version we do our own permission control - in this case $id doesn't matter
    if (!pnSecAuthAction(0, 'pagesetter::', "$tid:$pid:", ACCESS_READ))
      return pagesetterErrorApi(__FILE__, __LINE__, _PGNOAUTH);

    $hitCount =  pnModAPIFunc( 'pagesetter',
                               'user',
                               'getHitCount',
                               array('tid'    => $tid,
                                     'pid'    => $pid) );
  }
  //echo "$templateFile,$cacheID "; exit(0);

  $pubFormatted = $smarty->fetch($templateFile, $cacheID);

    // Hit count insertion
  $pubFormatted = str_replace('$hitCount$', $hitCount, $pubFormatted);

    // Check access to "edit this"
    // "edit this" insertion depending on permissions
  $isOwner = $pub['core']['creatorID'] == pnUserGetVar('uid');
  if (pnSecAuthAction(0, 'pagesetter::', "$tid:$pid:", pagesetterAccessEditor) ||
      $isOwner)
  {
    $editThisHTML = pagesetterGenerateEditThisHTML($pub,$tid,$pid,$id); // "<a href=\"$editThisURL\">" . _PGEDITTHIS . "</a>";

    $pubFormatted = str_replace('$editThis$', $editThisHTML, $pubFormatted);
  }
  else
    $pubFormatted = str_replace('$editThis$', '', $pubFormatted);

    // Make sure normal users do see this version cached
    // FIXME: concurrency problem here - what if the users access it during the code above?
  if (isset($id))
    pagesetterSmartyClearCache($tid, $pid);

  if ($updateHitCount)
    if (!pnModAPIFunc( 'pagesetter', 'user', 'incrementHitCount', array('tid' => $tid, 'pid' => $pid)))
      return false;

  return $pubFormatted;
}


function pagesetter_userapi_getPubArrayFormatted($args)
{
  if (!isset($args['tid'])  ||  $args['tid']=='')
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'tid' in 'pagesetter_userapi_getPubArrayFormatted'");

  $tid             = $args['tid'];
  $pubList         = $args['pubList'];
  $format          = (isset($args['format']) ? $args['format'] : 'list-single');
  $coreExtra       = (isset($args['coreExtra']) ? $args['coreExtra'] : array());
  $updateHitCount  = (isset($args['updateHitCount']) ? $args['updateHitCount'] : false);

  if (!pnModAPILoad('pagesetter', 'admin'))
    return pagesetterErrorApi(__FILE__, __LINE__, 'Failed to load Pagesetter admin API');
  
  $smarty = new pnRender('pagesetter'); // Used in blocks, so supply module name.
  $smarty->caching = false;
  $templateFile = pagesetterSmartyGetTemplateFilename($smarty, $tid, $format);
    
  // get pub type info
  $pubTypeInfo =  pnModAPIFunc( 'pagesetter', 'admin', 'getPubTypeInfo',
                                  array('tid' => $tid) );
  if ($pubTypeInfo === false)
    return false;

  // get detailed information on each publication
  foreach ($pubList['publications'] as $pub)
  {
    // get publication information
    $pubInfo = pnModAPIFunc('pagesetter', 
                            'user', 
                            'getPub', 
                            array('tid'    => $tid, 
                                  'pid'    => $pub['pid'],
                                  'format' => 'user') );
    // Escape necessary values
    foreach ($pubInfo as $key => $value)
      if (is_string($value))
        $pubInfo[$key] = pnVarPrepHTMLDisplay($value);
    
    // add print this, send this, and full URLs
    $printThisURL = pnModUrl('pagesetter','user','printpub',
                             array('tid'    => $tid,
                                   'pid'    => $pid));
    $printThisHTML = "<a href=\"$printThisURL\">" . _PGPRINTTHIS . "</a>";
    $pubInfo['core']['printThisURL'] = $printThisURL;
    $pubInfo['core']['printThis'] = $printThisHTML;

    $sendThisURL = pnModUrl('pagesetter','user','sendpub',
                            array('tid'    => $tid,
                                  'pid'    => $pid));
    $sendThisHTML = "<a href=\"$sendThisURL\">" . _PGSENDTHIS . "</a>";
    $pubInfo['core']['sendThisURL'] = $sendThisURL;
    $pubInfo['core']['sendThis'] = $sendThisHTML;


    $fullURL = pnModUrl('pagesetter','user','viewpub',
                        array('tid'    => $tid,
                              'pid'    => $pid));
    $pubInfo['core']['fullURL'] = $fullURL;

    // May perhaps not be specified on URL
    $pid = $pub['core']['pid'];
    $id  = $pub['core']['id'];

    // Add additional core values from caller
    $pubInfo['core'] += $coreExtra;

    // if the publication has information, add it to the array of publications.
    if (!($pubInfo === false))
      $pubsArray[] = $pubInfo;
    
    // update hit counts if required
    if ($updateHitCount)
      if (!pnModAPIFunc( 'pagesetter', 'user', 'incrementHitCount', array('tid' => $tid, 'pid' => $pid)))
        return false;
  }

  //echo "<br>pubsArray = "; print_r($pubsArray);
  $smarty->assign('publications', $pubsArray);

  if (!$smarty->template_exists($templateFile))
    return _PGSORRYNOTEMPLATE . ": $templateFile<p>";

  $allFormatted = $smarty->fetch($templateFile);

  return $allFormatted;
}


function pagesetterGenerateEditThisHTML(&$pub, $tid, $pid, $id)
{
  $html = '';

  $html .= "<script type=\"text/javascript\" src=\"modules/pagesetter/pnjavascript/editthis.js\"></script>\n";
  $html .= "<script type=\"text/javascript\" src=\"modules/pagesetter/guppy/psmenu.js\"></script>\n";
  $html .= "<a href=\"\" onclick=\"handleOnClickEditThis(this,event); return false\">" . _PGEDITTHIS . "</a>\n";

  $editThisURL = pnModUrl('pagesetter','user','pubedit',
                          array('tid'    => $tid,
                                'id'     => $id,
                                'action' => 'edit',
                                'goback' => '1'));
  
  $newURL = pnModUrl('pagesetter', 'user', 'pubedit',
                     array('tid' => $tid));

  $updatedDate = strftime(_PGDATEFORMAT, $pub['core']['lastUpdated']);
  $createdDate = strftime(_PGDATEFORMAT, $pub['core']['created']);

  $html .= "<table style=\"position: absolute; visibility: hidden;\" id=\"pubInfoBox\" onmouseout=\"psmenu.onMouseOutDiv(this)\" onmouseover=\"psmenu.cancelDelayedCloseMenu()\">\n";

  $html .=   "<tr><td>" . _PGFTPUBTITLE . ":</td><td>" . pnVarPrepHTMLDisplay($pub['core']['title']) . "</td></tr>\n"
           . "<tr><td>" . _PGAUTHOR . ":</td><td>" . pnVarPrepHTMLDisplay($pub['core']['author']) . "</td></tr>\n"
           . "<tr><td>" . _PGFTUPDATEDDATE . ":</td><td>$updatedDate</td></tr>\n"
           . "<tr><td>" . _PGFTCREATEDDATE . ":</td><td>$createdDate</td></tr>\n"
           . "<tr><td colspan=\"2\">\n"
           . "<a href=\"$editThisURL\">" . _PGEDITTHIS . "</a>\n"
           . " | <a href=\"$newURL\">" . _PGMENUPUBNEW . "</a>\n"
           . "</td></tr>\n"
           . "</table>\n";

  return $html;
}


function pagesetter_userapi_getHitCount($args)
{
  if (!isset($args['tid'])  ||  $args['tid']=='')
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'tid' in 'pagesetter_userapi_getHitCount'");
  if (!isset($args['pid'])  ||  $args['pid']=='')
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'pid' in 'pagesetter_userapi_getHitCount'");

  $tid = $args['tid'];
  $pid = $args['pid'];

  // No reason to check permissions - nobody cares about the hitcount alone. So no need for the overhead.

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $pubHeaderTable = $pntable['pagesetter_pubheader'];
  $pubHeaderColumn = $pntable['pagesetter_pubheader_column'];

  $sql = "SELECT $pubHeaderColumn[hitCount]
          FROM   $pubHeaderTable 
          WHERE  $pubHeaderColumn[pid] = '" . pnVarPrepForStore($pid) . "' AND
                 $pubHeaderColumn[tid] = '" . pnVarPrepForStore($tid) . "'";
  
  //echo "<pre>$sql</pre>"; exit(0);
  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return pagesetterErrorApi(__FILE__, __LINE__, '"getHitCount" failed: ' 
                                                  . $dbconn->errorMsg() . " while executing: <pre>$sql</pre>"); 
  $hitCount = $result->fields[0];
  $result->Close();

  return $hitCount;
}


function pagesetter_userapi_incrementHitCount($args)
{
  if (!isset($args['tid']))
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'tid' in 'pagesetter_userapi_incrementHitCount'");
  if (!isset($args['pid']))
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'pid' in 'pagesetter_userapi_incrementHitCount'");

  $tid = $args['tid'];
  $pid = $args['pid'];

    // Check access
  if (!pnSecAuthAction(0, 'pagesetter::', "$tid:$pid:", ACCESS_READ))
    return pagesetterErrorApi(__FILE__, __LINE__, _PGNOAUTH);

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $pubHeaderTable = $pntable['pagesetter_pubheader'];
  $pubHeaderColumn = $pntable['pagesetter_pubheader_column'];

  $sql = "UPDATE $pubHeaderTable SET
            $pubHeaderColumn[hitCount] = $pubHeaderColumn[hitCount] + 1
          WHERE $pubHeaderColumn[pid] = '" . pnVarPrepForStore($pid) . "' AND
                $pubHeaderColumn[tid] = '" . pnVarPrepForStore($tid) . "'";
  
  //echo "<pre>$sql</pre>"; exit(0);
  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return pagesetterErrorApi(__FILE__, __LINE__, '"incrementHitCount" failed: ' 
                                                  . $dbconn->errorMsg() . " while executing: <pre>$sql</pre>"); 

  return true;
}


function pagesetter_userapi_parseFilter($args)
{
  if (!isset($args['tid']))
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'tid' in 'pagesetter_userapi_parseFilter'");
  if (!isset($args['filter']))
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'filter' in 'pagesetter_userapi_ParseFilter'");

  $tid          = $args['tid'];
  $filterStrSet = $args['filter'];

  if (!pnModAPILoad('pagesetter', 'admin'))
    return pagesetterErrorApi(__FILE__, __LINE__, 'Failed to load Pagesetter admin API');

  $pubInfo =  pnModAPIFunc( 'pagesetter',
                            'admin',
                            'getPubTypeInfo',
                            array('tid' => $tid) );

  $fields        = $pubInfo['fields'];
  $fieldIndexMap = $pubInfo['fieldIndex'];

  $pntable        = pnDBGetTables();
  $pubColumn      = $pntable['pagesetter_pubdata_column'];
  $listItemsTable = $pntable['pagesetter_listitems'];

    // Iterate through each of the filter strings (the disjunctions)

  $firstFilter = true;
  $sql = '';
  $join = '';
  $listTableCount = 1;
  $joinedListFields = array();

  foreach ($filterStrSet as $filterStr)
  {
      // Split filter string into terms and iterate over those

    $terms = preg_split("/\s*,\s*/", $filterStr);

    $firstTerm = true;

    if (!$firstFilter)
      $sql .= ' OR ';
    $firstFilter = false;

    foreach ($terms as $termStr)
    {
        // Split term into operands

      $atoms = preg_split("/\s*:\s*/", $termStr);

      if (count($atoms) >= 2)
      {
        $fieldName = $atoms[0];
        $operator  = $atoms[1];
        $value     = $atoms[2];

          // Check field names and convert to column names

        if (substr($fieldName,0,5) == 'core.')
        {
          $columnName = $pubColumn[substr($fieldName,5)];
          if (!isset($columnName))
            return pagesetterErrorApi(__FILE__, __LINE__, "Unknown field name '$fieldName'");
        }
        else
        {
          $fieldIndex = $fieldIndexMap[$fieldName];
          if (!isset($fieldIndex))
            return pagesetterErrorApi(__FILE__, __LINE__, "Unknown field name '$fieldName'");

          $fieldID = $fields[$fieldIndex]['id'];
          $columnName = pagesetterGetPubColumnName($fieldID);
        }

        if (!$firstTerm)
          $sql .= ' AND ';
        $firstTerm = false;

        if ($operator == 'sub')
        {
          if ($value != 'top')
          {
            if (!isset($joinedListFields[$columnName]))
              $join .= "\nLEFT JOIN $listItemsTable as {$columnName}_table\n"
                      .   " ON {$columnName}_table.pg_id = $columnName";

            $join .= "\nLEFT JOIN $listItemsTable as listTable{$listTableCount}\n"
                    .   " ON listTable{$listTableCount}.pg_id = '" . pnVarPrepForStore($value) . "'";
            
            $joinedListFields[$columnName] = 1;

            $sql .= "({$columnName}_table.pg_lval >= listTable{$listTableCount}.pg_lval AND {$columnName}_table.pg_rval <= listTable{$listTableCount}.pg_rval)";

            ++$listTableCount;
          }
          else
            $sql .=" 1=1";
        }
        else if ($operator == 'like')
        {
          $sql .= "$columnName LIKE '%" . pnVarPrepForStore($value) . "%'";
        }
        else
        {

          $sql .= $columnName;

          switch ($operator)
          {
            case 'eq':      $sql .= ' = '; break;
            case 'ne':      $sql .= ' != '; break;
            case 'lt':      $sql .= ' < '; break;
            case 'gt':      $sql .= ' > '; break;
            case 'le':      $sql .= ' <= '; break;
            case 'ge':      $sql .= ' >= '; break;
            case 'null':    $sql .= ' is null'; break;
            case 'notnull': $sql .= ' is not null'; break;

            default: return pagesetterErrorApi(__FILE__, __LINE__, "Unknown filter operator '$operator'.");
          }

          if ($value == '@now')
            $value = date('Y-m-d');

          if ($operator != 'null'  &&  $operator != 'notnull')
            $sql .= "'" . pnVarPrepForStore($value) . "'";
        }
      }
    }
  }

  //echo "<pre>JOIN:\n$join\nSQL:\n$sql</pre>\n"; exit(0);
  
  return array( 'sql'  => $sql,
                'join' => $join );
}


function pagesetter_userapi_parseOrderBy($args)
{
  if (!isset($args['tid']))
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'tid' in 'pagesetter_userapi_parseOrderBy'");
  if (!isset($args['orderByStr']))
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'orderByStr' in 'pagesetter_userapi_ParseOrderBy'");

  $tid        = $args['tid'];
  $orderByStr = $args['orderByStr'];

  if (!pnModAPILoad('pagesetter', 'admin'))
    return pagesetterErrorApi(__FILE__, __LINE__, 'Failed to load Pagesetter admin API');

  $pubInfo =  pnModAPIFunc( 'pagesetter',
                            'admin',
                            'getPubTypeInfo',
                            array('tid' => $tid) );

  $fields        = $pubInfo['fields'];
  $fieldIndexMap = $pubInfo['fieldIndex'];

  $pntable         = &pnDBGetTables();
  $pubColumn       = $pntable['pagesetter_pubdata_column'];
  $pubHeaderTable  = $pntable['pagesetter_pubheader'];
  $pubHeaderColumn = $pntable['pagesetter_pubheader_column'];

  $orderBy = array();

    // Split orderBy statement by commas and iterate over those
  $terms = preg_split("/\s*,\s*/", $orderByStr);
  $first = true;

  foreach ($terms as $termStr)
  {
      // Split term into field and modifer
    $atoms = preg_split("/\s*:\s*/", $termStr);

    if (count($atoms) > 0)
    {
      $fieldName = $atoms[0];
      $desc      = false;

      if (count($atoms) > 1)
      {
        if ($atoms[1] == 'desc')
          $desc = true;
      }

        // Convert field name to column ID
      if ($fieldName == 'core.hitCount')
      {
        $columnName = $pubHeaderTable . '.' . $pubHeaderColumn['hitCount'];
      }
      else if (substr($fieldName,0,5) == 'core.')
      {
        $columnName = $pubColumn[substr($fieldName,5)];
        if (!isset($columnName))
          return pagesetterErrorApi(__FILE__, __LINE__, "Unknown field name '$fieldName'");
      }
      else
      {
        $fieldIndex = $fieldIndexMap[$fieldName];
        if (!isset($fieldIndex))
          return pagesetterErrorApi(__FILE__, __LINE__, "Unknown field name '$fieldName'");

        $fieldID = $fields[$fieldIndex]['id'];
        $columnName = pagesetterGetPubColumnName($fieldID);
      }

      $orderBy[] = array( 'name' => $columnName,
                          'desc' => $desc );
    }
  }

  return $orderBy;
}


function pagesetter_userapi_search(&$args)
{
  if (!isset($args['query']))
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'query' in 'pagesetter_userapi_search'");
  if (!isset($args['callback']))
    return pagesetterErrorApi(__FILE__, __LINE__, "Missing argument 'callback' in 'pagesetter_userapi_search'");

  $query    = $args['query'];
  $match    = $args['match'];
  $callback = &$args['callback'];

  if (!pnModAPILoad('pagesetter', 'admin'))
    return pagesetterErrorApi(__FILE__, __LINE__, 'Failed to load Pagesetter admin API');

  list($dbconn) = pnDBGetConn();
  $pntable      = pnDBGetTables();

    // Split query by whitespace
  $words = preg_split("/\s+/", $query);

  $pubTypes = pnModAPIFunc( 'pagesetter',
                            'admin',
                            'getPublicationTypes' );

  $callback->start();

    // Iterate through all publication types
  foreach ($pubTypes as $pubType)
  {
    $tid = $pubType['id'];

    $searchableColumns = pnModAPIFunc( 'pagesetter',
                                       'admin',
                                       'getSearchableColumns',
                                       array('tid' => $tid) );

    if (count($searchableColumns) > 0)
    {
      $pubInfo = pnModAPIFunc( 'pagesetter',
                               'admin',
                               'getPubTypeInfo',
                               array('tid' => $tid) );

      $titleFieldID = $pubInfo['publication']['titleFieldID'];
      $titleColumn = pagesetterGetPubColumnName($titleFieldID);
      $pubTitle = $pubInfo['publication']['title'];

      $where = '';
      foreach ($searchableColumns as $column)
      {
        if ($where != '')
          $where .= " OR ";

        $where .= "(";

        $firstWord = true;
        foreach ($words as $word)
        {
          if (!$firstWord)
            $where .= ($match == 'AND' ? ' AND ' : ' OR ');
          $firstWord = false;

          $where .= "$column LIKE '%" . pnVarPrepForStore($word) . "%'";
        }

        $where .= ")";
      }

      $pubTable = pagesetterGetPubTableName($tid);
      $pubColumn = $pntable['pagesetter_pubdata_column'];

      if ($where != '')
        $where = "AND ($where)";

      $sql = "SELECT
                $pubColumn[pid],
                $titleColumn
              FROM
                $pubTable
              WHERE
                    ($pubColumn[publishDate] <= CURDATE() || $pubColumn[publishDate] IS NULL)
                AND (CURDATE() < $pubColumn[expireDate] || $pubColumn[expireDate] IS NULL)
                AND NOT $pubColumn[inDepot]
                AND $pubColumn[online]
                $where
              ORDER BY $titleColumn";

      //echo "<pre>$sql</pre>\n"; exit(0);

      $result = $dbconn->execute($sql);

      if ($dbconn->errorNo() != 0) // FIXME, not correct
        return pagesetterErrorApi(__FILE__, __LINE__, '"search" failed: ' 
                                                      . $dbconn->errorMsg() . " while executing: $sql");

      for (; !$result->EOF; $result->MoveNext())
      {
        $pid   = $result->fields[0];
        $title = $result->fields[1];

        if (pnSecAuthAction(0, 'pagesetter::', "$tid:$pid:", ACCESS_READ))
        {
          $url = pnModUrl('pagesetter','user', 'viewpub',
                          array('tid'    => $tid,
                                'pid'    => $pid));


          $callback->found($tid, $pid, $pubTitle, $title, $url);
        }
      }
  
      $result->Close();
    }
  }

  $callback->stop();

  return true;
}


?>
